#ifndef DSR_BIPARTITE_MATCHING_H
#define DSR_BIPARTITE_MATCHING_H

#include <vector>
#include <limits>
#include <boost/graph/adjacency_list.hpp>
#include <fstream>


namespace bipartite_matching_detail {
  //template <class ItE, class ItW, class OItP>
  typedef std::pair<int,int> Edge;
  typedef double Weight;
  typedef std::vector<Edge > Edges;
  typedef Edges Out;
  typedef std::vector<double> Weights;
  typedef std::vector<bool> Prime;
  typedef std::pair<int, Weight> WeightedEdge;
  typedef std::vector<WeightedEdge> Neighbors;
  typedef std::vector<WeightedEdge> Distances;
  typedef std::vector<Neighbors> UAdjacency;
  typedef std::vector<WeightedEdge> VAdjacency;

  inline Edge make_edge(int u, int v){
    return Edge(u,v);
  }

 
  inline void flip_edge(int u, int v, UAdjacency &ua, VAdjacency &va){
    // add edge u,v to the matching
    for (unsigned int i=0; i< ua[u].size(); ++i){
      if (ua[u][i].first==v){
        Weight w= ua[u][i].second;
	  
        ua[u].erase(ua[u].begin() + i);
        assert(va[v].first == -1);
        va[v]=WeightedEdge(u, w);
        return;
      }
    }
    assert(0);
  }
  inline void flip_edge(int v, int u, VAdjacency &va, UAdjacency &ua){
    // remove edge u,v from the matching
    assert(va[v].first == u);
    va[v].first = -1;
    ua[u].push_back(WeightedEdge(v, va[v].second));
    va[v].second= Weight(-1);
  }

  inline void relax(int u,
                    Weight udist,
                    const Neighbors &uadj,
                    Distances &vdists){
    for (unsigned int i=0; i< uadj.size(); ++i){
      int v= uadj[i].first;
      // negative since we flip that edge to out of the set
      Weight dist= udist - uadj[i].second;
      if (vdists[v].second > dist) {
        vdists[v].first= u;
        vdists[v].second= dist;
      }
    }
  }
 
  inline void relax(int v,
                    Weight vdist,
                    const WeightedEdge &vadj,
                    Distances &udists){
    if (vadj.first == -1) return; // there is no incoming edge
    int u= vadj.first;
    Weight ndist = vdist + vadj.second;
    if (ndist < udists[u].second){
      udists[u] = WeightedEdge(v, ndist);
    }
  }


  inline void find_augmenting_path(const Prime &up, const Prime &vp, 
                                   const UAdjacency& uadj, const VAdjacency &vadj,
                                   std::back_insert_iterator<Edges > out){
	//fprintf(stderr,"find augmenting path\n");
    if (up.empty() || vp.empty()) return;
    Distances u_dists(uadj.size());
    Distances v_dists(vadj.size());
	//fprintf(stderr,"for\n");
    for (unsigned int i=0; i< u_dists.size(); ++i){
      if (!up[i]){
        u_dists[i]=WeightedEdge(-1, std::numeric_limits<Weight>::max());
      } else {
        u_dists[i]=WeightedEdge(-1, 0);
      }
    }
	//fprintf(stderr,"for2\n");
    for (unsigned int i=0; i< v_dists.size(); ++i){
      v_dists[i]=WeightedEdge(-1, std::numeric_limits<Weight>::max());
    }
//fprintf(stderr,"for3\n");
    for (unsigned int i=0; i< uadj.size() + vadj.size()-1; ++i){
      for (unsigned int j=0; j< u_dists.size(); ++j){
        relax(j, u_dists[j].second, uadj[j], v_dists);
      }
      for (unsigned int j=0; j< v_dists.size(); ++j){
        relax(j, v_dists[j].second, vadj[j], u_dists);
      }
    }
//fprintf(stderr,"extract path\n");
    // extract path
    Weight mind= std::numeric_limits<Weight>::max();
    int minv(-1);
    for (unsigned int v=0; v< vp.size(); ++v){
      if (vp[v] && v_dists[v].second < mind){
        mind= v_dists[v].second;
        minv= v;
      }
    }
	//fprintf(stderr,"if\n");
    if (minv==-1){
      // we did not succeed in connected the two, so we are done.
	  //fprintf(stderr,"fail\n");
      return;
    } else {
	//fprintf(stderr,"succeed\n");
      int curu= v_dists[minv].first;
      std::pair<int,int> ret(curu, minv);
      *out= ret;
      ++out;
      while (u_dists[curu].first != -1){
        int pred= u_dists[curu].first;
        curu= v_dists[pred].first;
        std::pair<int,int> ret(curu, pred);
        *out= ret;
        ++out;
      }
    }
	//fprintf(stderr,"r\n");
	return;
  }

  inline void compute(UAdjacency &uadj,
                      VAdjacency &vadj,
                      Out& out){
    // make U,V prime
	//fprintf(stderr,"prime\n");
    Prime up(uadj.size(), true);
    Prime vp(vadj.size(), true);
    unsigned int remaining_ind= std::min(uadj.size(), vadj.size());

    while (remaining_ind != 0) {
      //fprintf(stderr,"remaining %d\n",remaining_ind);
      std::vector<std::pair<int,int> > path;
      // this returns the path in reverse order,
      // it does not matter currently since the length is always odd
      find_augmenting_path(up, vp, uadj, vadj, std::back_inserter(path));
	  //fprintf(stderr,"Hmm\n");
      //fprintf(stderr,"path is [");
      //for (unsigned int i=0; i< path.size(); ++i) {
      //  std::cerr << " (" << path[i].first << ", " << path[i].second << ")";
      //}
      //fprintf(stderr,"]\n");
      if (!path.empty()){
          //fprintf(stderr,"Found path %d: ",(unsigned) path.size());
          //for (unsigned int i=path.size()-1; i< path.size(); --i){
          //std::cerr << "u" << path[i].first << " v" << path[i].second << " ";
          //}
          //fprintf(stderr,"\n");

        for (unsigned int i=0; i+1< path.size(); ++i){
          // flip all edges out of match.Looks strange because the path is returned backwards
          flip_edge(path[i+1].second, path[i].first, vadj, uadj);
        }
        for (unsigned int i=0; i< path.size(); ++i){
          // flip all edges into of match
          flip_edge(path[i].first, path[i].second, uadj, vadj);
        }
        vp[path.front().second]=false;
        up[path.back().first]=false;
        --remaining_ind;

        //fprintf(stderr, "Matching is ");
          /*for (unsigned int i=0; i< vadj.size(); ++i){
          V v(i);
          if (vadj[i].first != -1){
          std::cerr <<  "(" << vadj[v].first<< "-" << v << ") ";
          }
          }*/
         // fprintf(stderr,"\n");
      } else {
        break;
      }
    }
//fprintf(stderr,"extract meaning\n");
    // extract matching
    for (unsigned int i=0; i< vadj.size(); ++i){
      int v(i);
      if (vadj[i].first != -1){
        out.push_back( make_edge(vadj[v].first, v) );
      }
    }
//fprintf(stderr,"return\n");
  }

};



/** Compute the maximum weight bipartite matching on a bipartite
    graph.

    \param[in] g A boost edge and vertex list bipartite graph
    \param[in] na Vertices [0,na) must be in the first set.
    \param[in] im The vertex index property map for the graph.
    \param[in] wm The edge weight property map for the graph.

    \return a list of pairs of vertex indexes for the edges
    connecting the two sets. In each pair the first is in
    [0,na) and the second in [na,boost::num_vertices(g)).
*/
template <class BoostGraph, class IndexMap, class WeightMap>
inline std::vector<std::pair< int, int> >
get_maximum_weight_bipartite_matching(const BoostGraph &g,
                                      unsigned int na,
                                      const IndexMap &im,
                                      const WeightMap &wm) {
  std::vector<std::pair<int, int> > out;
  // copy graph
  //fprintf(stderr,"a\n");
  typedef typename boost::graph_traits<BoostGraph>::edge_iterator EI;
  std::pair<EI, EI> edges=boost::edges(g);


  //fprintf(stderr,"b\n");
  typename bipartite_matching_detail::UAdjacency uadj(na);
  typename bipartite_matching_detail::VAdjacency vadj(boost::num_vertices(g)-na,
        bipartite_matching_detail::WeightedEdge(-1,
		std::numeric_limits<double>::max()));
  //fprintf(stderr,"c\n"); 
  for (; edges.first != edges.second; ++edges.first){
    int s= im[boost::source(*edges.first, g)];
    int t= im[boost::target(*edges.first, g)];
    if (s> t) std::swap(s,t);
    double w= wm[*edges.first];

  //fprintf(stderr,"d\n");

    uadj[s].push_back(bipartite_matching_detail::WeightedEdge(t-na, w));
  }

  //fprintf(stderr,"e\n");
  bipartite_matching_detail::compute(uadj, vadj, out);
  //fprintf(stderr,"f\n");
  typename std::vector<typename std::pair<int, int> >
    ret(out.size());
  for (unsigned int i=0; i< out.size(); ++i) {
    // assume vertices are indices
    ret[i]= std::make_pair(out[i].first, out[i].second+na);
  }
  //fprintf(stderr,"g\n");
  return ret;
}

#endif // DSR_BIPARTITE_MATCHING_H
