#include "graph.h"
#include "utilities.h"
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <queue>
#include <time.h>
#include <algorithm>
#include <math.h>
#include <fstream>	

#include <pthread.h>

#include <boost/config.hpp>
#include <boost/utility.hpp>             // for boost::tie
#include <boost/graph/graph_traits.hpp>  // for boost::graph_traits
#include <boost/graph/adjacency_list.hpp>
#include <boost/graph/max_cardinality_matching.hpp> //For Edmonds matching algorithm
#include <boost/property_map/property_map.hpp> //for matemap?
#include <boost/concept_check.hpp>
#include "dsrBipartiteMatching.h"

/*** Constructors ***/
//initializes an empty graph such that this->directed = directed, num_nodes = 0, and max_id = 0
//sets uptodatePartitions to true, uptodateIndices to true
MyGraph::MyGraph(bool directed)
{
	this->directed = directed;
	this->num_edges = 0; 
	this->max_id = 0;
	this->uptodateIndices = true;
	this->uptodatePartitions = true;
	//bGraph= new boost::adjacency_list<boost::vecS, boost::vecS, boost::bidirectionalS>();
}


//reads nodes from a file, if edges = true graph will add edges from the file. these edges will be directed/undirected based on the last argument
//this sets i-degrees of nodes according to degreeArg
MyGraph::MyGraph(std::string file, bool edges, positioningArgument& degreeArg, bool directed, char partitionMethod,bool plain,struct arguments& args)
{
	this->args=&args;
	this->directed = directed;
	this->num_edges = 0; 
	this->max_id = 0;
	this->uptodateIndices = true;
	this->uptodatePartitions = true;
	args.T.start("Read Graph");
	if((!plain && this->readGraph(file.c_str(), edges, args)==2)||(plain && this->readPlainGraph(file.c_str(), edges, args)==2))
	{
		fprintf(stderr,"File (\"%s\") Failed To Open, exiting program\n", file.c_str());
		exit(1);
	}
	args.T.stop("Read Graph");
	initIndex(partitionMethod,degreeArg,args);
}

/*** Init ***/
void MyGraph::initIndex(char partitionMethod,positioningArgument& degreeArg, arguments& args)
{
	//fprintf(stderr,"partitionMethod int = %d or aka as a char= %c\n",partitionMethod,partitionMethod);
	if(true||partitionMethod=='N'
		|| partitionMethod=='i')//use the new iterative partition method
	{
		args.T.start("indexing");
		nodeSorted = std::vector<struct MyNode*>(nodes.size(),NULL);
		unsigned int count=0;
		for(mapIDNodes::iterator i=nodes.begin(); i!=nodes.end();i++)
		{
			nodeSorted[count]=&(i->second);
			count++;
		}//note, nodes are sorted here by their id number, not by anything else.
		indexing.init(nodeSorted,args);
		this->uptodatePartitions = true;
		args.T.stop("indexing");
	}
	else
	{
		fprintf(stderr,"PartitionMethod Unknown %d\n",partitionMethod);
		exit(0);
	}
}
void MyGraph::createNodeSorted()
{
nodeSorted = std::vector<struct MyNode*>(nodes.size(),NULL);
unsigned int count=0;
	for(mapIDNodes::iterator i=nodes.begin(); i!=nodes.end();i++)
	{
		nodeSorted[count]=&(i->second);
		count++;
	}
	sort(nodeSorted.begin(),nodeSorted.end(), lexFirst); //lexFirst is found in utilities
	
}
//initializes partitions: sorts nodes in the vector nodeSorted lexicographically by i-degrees 
//then iterates through node sorted and fills in the partitionStarts[0-3] where partitionsStarts[i] is the start of each new partition

/*** File I/O ***/
//reads from a file, depending on degreeArg either i-degrees will be calculated or read from a file
int MyGraph::readGraph(const char* file, bool edges,arguments& args)
{
	fprintf(stdout,"Use read plain graph\n");
	
	return readPlainGraph(file, edges,args);
}


int MyGraph::readPlainGraph(const char* file, bool edges,arguments& args)
{
	unsigned int i, id1, id2, num_nodes;
	std::string label;
	unsigned short type;
	std::ifstream input(file);
	if (input == NULL) 
	{
		perror("Could not open the specified file in read mode.");
		printf("errno = %d.\n",errno);
		return 2;
	}
	input>>num_nodes;
	fprintf(stderr, "read in num_nodes = %u\n", num_nodes);
	std::string line;std::getline(input, line);
	for(i = 0; i < num_nodes; i++) 
	{
		input>>id1>>label>>type;
		this->addNode(label, type, id1);
	}
	num_edges = 0;
	//read edges now
	if(edges){
		input>>id1;
		while(!input.eof())
		{
			input>>id2>>label;
			if(!args.useEdgeLabels)
			{
				label="";
			}
			this->addEdge(id1, id2,label);
			input>>id1;
		}
	}
	args.T.stop("Read Graph");
	args.T.print(stderr);
	calculateBasicStatistics();
	return 0;
}

void MyGraph::calculateBasicStatistics()
{
	//iterate through nodes to calculate statistics
	unsigned long max_edges_per_node = 0, avg_edges_per_node = 0, num_nodes_with_many_edges = 0, num_nodes_with_few_edges = 0;
	struct MyNode * biggestNode = &(this->nodes.begin()->second); 
	for (mapIDNodes::iterator iter = nodes.begin(); iter != nodes.end(); iter++) 
	{
		edgelist* outedges=iter->second.getOutEdges();
		avg_edges_per_node+=outedges->size();
		if (outedges->size() > max_edges_per_node) 
		{
			max_edges_per_node = outedges->size();
			biggestNode = &iter->second;
		}
		if (outedges->size() >= MANY_EDGES) {
			//fprintf(stdout,"\tStar: %s\n",iter->second->getLabel());
			num_nodes_with_many_edges++;
		}
		if (outedges->size() <= FEW_EDGES) {

			num_nodes_with_few_edges++;
		}
	}
	fprintf(stderr,"\nbiggest= %u %s %u\n",(unsigned)biggestNode->getID(), biggestNode->getLabel(), (unsigned)biggestNode->getNumOutNeighbors());
	
	fprintf(stderr,"Total Nodes: %u; Max edges per node: %lu; Average edges per node: %f; Num nodes with many edges (> %d): %lu; Num nodes with few edges (<= %d): %lu\n",getNumNodes(),max_edges_per_node, ((double) ((double) avg_edges_per_node / (double) nodes.size())), MANY_EDGES, num_nodes_with_many_edges, FEW_EDGES, num_nodes_with_few_edges );
	fflush(stderr);
}

MyGraph* MyGraph::generateFragment_1N(const MyNode* center,const arguments& args)
{
	MyGraph * frag = new MyGraph(this->isDirected());
	//add central node
	frag->addNode(center,false);

	const edgelist* potentialEdges  = center->getOutEdgesConst();
	for(edgelist::const_iterator i=potentialEdges->begin(); i !=potentialEdges->end(); ++i)
	{
		frag->addNode(i->second.getNode(),true);
	}
	frag->updatePositions(1,args);
	return frag;
}

//writes a graph to the file <file>.grf
int MyGraph::writeGraph(const char* file)const
{
	unsigned int i;
	FILE * output;
	mapIDNodes::const_iterator it;

	output = fopen(file, "wb");
	if (output == NULL) 
	{
		perror("Could not create or open file in write mode.");
		printf("errno = %d.\n",errno);
		return 2;
	}
	fprintf(stdout,"numnodes\n");
	unsigned int num_nodes = this->getNumNodes();
	fwrite((const void*) &(num_nodes),sizeof(unsigned int),1,output);
	fprintf(stdout,"a %d\n",num_nodes);
	for (it=nodes.begin(); it != nodes.end(); it++) 
	{//fprintf(stdout,"b\n");
		fwrite((const void*) (it->second).getID(),sizeof(unsigned int),1,output);
		//fprintf(stdout,"c\n");
		fwrite((const void*) (it->second).getLabelLen(),sizeof(unsigned char),1,output);
	//fprintf(stdout,"d\n");
		fprintf(output,"%s", (it->second).getLabel());
		//fprintf(stdout,"d2\n");
		fwrite((const void*) (it->second).getType(),sizeof(unsigned short),1,output);
	}

	//fprintf(stdout,"e\n");
	for (it=nodes.begin(); it != nodes.end(); it++) 
	{
		const edgelist* outedges=it->second.getOutEdgesConst();
		for (edgelist::const_iterator i=outedges->begin(); i !=outedges->end(); i++) 
		{
			if(this->isDirected() || it->second.getID() <i->first)
			{
				fwrite((const void*) it->second.getID(),sizeof(unsigned int),1,output);
				fwrite((const void*) i->first,sizeof(unsigned int),1,output);
			}
		}
	}
	fclose(output);
	return 0;
}

int MyGraph::writePlainGraph(const char* file)const
{
	std::ofstream output(file);
	mapIDNodes::const_iterator it;
	
	if (!output.is_open()) 
	{
		perror("Could not create or open file in write mode.");
		printf("errno = %d.\n",errno);
		return 2;
	}
	unsigned int num_nodes = this->getNumNodes();
	output<<num_nodes<<"\n";
	for (it=nodes.begin(); it != nodes.end(); it++) 
	{			
		output<<it->second.getID()<<" "<<(it->second).getLabel()<<" ";
		output<<it->second.getType()<<"\n";
	}

	for (it=nodes.begin(); it != nodes.end(); it++) 
	{
		const edgelist* outedges=it->second.getOutEdgesConst();
		for (edgelist::const_iterator i=outedges->begin(); i !=outedges->end(); i++) 
		{
			if(this->isDirected()|| it->second.getID() < i->first)
			{
				output<<(it->second.getID())<<" ";
				output<<i->first<<" ";
				output<<i->second.getEdgeLabel()<<"\n";
			}
		}
	}
	output.close();
	return 0;
}
//reads in the i-degrees from file
int MyGraph::readIdegrees(std::string file)
{
	
	unsigned int i, id1;
	std::ifstream input(file.c_str());
	MyNode *n;

	if (!input.is_open()) {
		perror("Could not open the specified file in read mode.");
		printf("errno = %d.\n",errno);
		return 2;
	}

	fprintf(stderr,"\nReading i-degrees from file (%s)\n", file.c_str());
	fflush(stderr);

	//read node data
	unsigned int num_nodes;
	input>>num_nodes;
	fprintf(stderr, "\n\tnum_nodes = %u\n", num_nodes);
	fflush(stderr);
	std::string next;
	for(i = 0; i < num_nodes; i++) 
	{
		input>>id1;
		n=this->getNodeById(id1);
		for(unsigned int i_deg = 1; i_deg <= 3; ++i_deg)
		{
			while(input>>next)
			{
				if(next==",")
					{break;}
				unsigned int t=atoi(next.c_str());
				float d;
				input>>d;
				n->getIDegreeVar(i_deg,false)->append(t,d);
			}
		}
		for(unsigned int i_deg = 1; i_deg <= 3; ++i_deg)
		{
			while(input>>next)
			{
				if(next==",")
					{break;}
				unsigned int t=atoi(next.c_str());
				float d;
				input>>d;
				n->getIDegreeVar(i_deg,true)->append(t,d);
			}
		}
	}

	input.close();
	this->uptodatePartitions = false;
	return 0;
}

//writes the i-degrees of the graph to the file
int MyGraph::writeIdegrees(const char* file)const
{
	FILE * f;
	mapIDNodes::const_iterator n;
	f = fopen(file, "w");
	if (f == NULL) 
	{
		perror("Could not create or open file in write mode.");
		printf("errno = %d.\n",errno);
		return 2;
	}
	//write node information
	unsigned int num_nodes = this->getNumNodes();
	fprintf(f,"%u\n",num_nodes);

	for (n=nodes.begin(); n != nodes.end(); n++) 
	{
		fprintf(f,"%u\n",(n->second).getID());
		for(unsigned int i = 0; i < 3; ++i)
		{
			iDegreeList::const_iterator itr=(n->second).getIDegree(i+1,false)->getList()->begin();
			for(; itr !=n->second.getIDegree(i+1,false)->getList()->end();itr++)
			{
				fprintf(f, "%u %u ", (unsigned int)itr->first,(unsigned int)itr->second);
			}
			fprintf(f, ",\n");
		}
		for(unsigned int i = 0; i < 3; ++i)
		{
			iDegreeList::const_iterator itr=n->second.getIDegree(i+1,true)->getList()->begin();
			for(; itr !=n->second.getIDegree(i+1,true)->getList()->end();itr++)
			{
				fprintf(f, "%u %u ", (unsigned int)itr->first,(unsigned int)itr->second);
			}
			fprintf(f, ",\n");
		}
	}

	fclose(f);
	//TODO FIX
	//Is there a function which reads this format?
	return 0;
	
}


/*** Accessors ***/
//returns the total number of nodes in the graph
unsigned int MyGraph::getNumNodes()const
{
	return this->nodes.size();
}

//returns the total number of edges
unsigned long MyGraph::getNumEdges()const
{
	return this->num_edges;
}
//returns a Node pointer based on the node id a null pointer is returned if the ID doesn't exist
MyNode* MyGraph::getNodeById(unsigned int id)
{
	mapIDNodes::iterator it = this->nodes.find(id);
	MyNode * n = NULL;
	if( it != this->nodes.end() )
	{
		n = &(it->second);
	}

	return n;
}
const MyNode* MyGraph::getNodeByIdConst(unsigned int id)const
{
	mapIDNodes::const_iterator it = this->nodes.find(id);
	const MyNode * n = NULL;
	if( it != this->nodes.end() )
	{
		n = &(it->second);
	}

	return n;
}
//returns a Node pointer based on the index, if the index is not within [0,numNodes) a null pointer is returned
MyNode* MyGraph::getNodeByIndex(unsigned int index)const
{
	MyNode * n = NULL;

	if(index < this->getNumNodes() && this->uptodatePartitions)
	{
		n = this->nodeSorted[index];
	}

	return n;
}

//returns the index of the node <ID>  if <ID> doesn't exit UINT_MAX is returned
unsigned int MyGraph::getIndexById(unsigned int id)const
{
	unsigned int index = UINT_MAX;

	if(this->uptodatePartitions)
	{
		for(unsigned int i = 0; i < this->nodeSorted.size()&& index == UINT_MAX; ++i)
		{
			if(this->nodeSorted[i]->getID() == id)
			{
				index = i;
			}
		}
	}

	return index;
}

//returns the nodes within <degree> of <id> 
std::vector<const struct MyNode *> MyGraph::getNeighborsToDegreeById(unsigned int id, unsigned int degree)const
{
	std::vector<const struct MyNode *> neighbors; 
	std::map<unsigned int, unsigned int> within = this->rBFS(id, degree, true);
	neighbors.reserve(within.size());

	for(std::map<unsigned int, unsigned int>::iterator it = within.begin(); 
		it!=within.end(); ++it)
	{
		if(it->first != id)
		{
			neighbors.push_back(this->getNodeByIdConst(it->first));
		}
	}

	return neighbors;
}

//returns the node in the graph with the largest number of neighbors
const struct MyNode* MyGraph::centerPopular() const
{
	const MyNode * n = NULL;
	const MyNode * next; 
	if(this->getNumNodes() > 0)
	{
		n = this->getNodeByIndex(0);
		for(unsigned int i = 0; i < this->getNumNodes(); ++i)
		{
			next = this->getNodeByIndex(i);
			if(next->getNumOutNeighbors() > n->getNumOutNeighbors() )
			{
				n = next;
			}
		}
	}

	return n;
}
//returns the node in the graph with the highest number of different neighbors (ties are broken by total number of neighbors)
const struct MyNode* MyGraph::centerDiverse() const
{
	const MyNode * n = NULL;
	if(this->getNumNodes() <= 0)
	{ 
		fprintf(stderr,"in centerDiverse: Graph does not have nodes\n");
		return n;
	}
	n = this->getNodeByIndex(0);
	//unsigned int max_neighbor_types = 0;
	//for each node
	for(unsigned int i = 1; i < this->getNumNodes(); ++i)
	{
		//calculate its diversity 
		if(this->isMoreDiverse(this->getNodeByIndex(i),n))
		{
			n = this->getNodeByIndex(i);
		}
	}

	return n;
}
//returns the node in the graph with the highest number of different neighbors (ties are broken by total number of neighbors)
const struct MyNode* MyGraph::centerDiverseTotal() const
{
	const MyNode * n = NULL;
	if(this->getNumNodes() <= 0)
	{ 
		fprintf(stderr,"in centerDiverse: Graph does not have nodes\n");
		return n;
	}
	n = this->getNodeByIndex(0);
	//unsigned int max_neighbor_types = 0;
	//for each node
	for(unsigned int i = 1; i < this->getNumNodes(); ++i)
	{
		//calculate its diversity 
		if(this->isMoreDiverseTotal(this->getNodeByIndex(i),n))
		{
			n = this->getNodeByIndex(i);
		}
	}

	return n;
}
bool MyGraph::isDirected() const
{
	return this->directed;
}

const MyNode * MyGraph::getOriginator() const
{
	return this->getNodeByIdConst(this->originator);
}

void MyGraph::setOriginator(unsigned int o)
{
	this->originator=o;
}

//gets the <degree> neighbor hood around <id> and forms the induced subgraph around it
//if skipPartitioning is true: i-degrees are not calculated otherwise: i-degrees are calculated based on the entire subgraph
MyGraph* MyGraph::getSubgraphToDegreeById(unsigned int id, unsigned char degree, bool skipPartitioning,const arguments& args,bool centerIdegree)
{
	MyGraph * g = new MyGraph(this->isDirected());
	const MyNode * n = this->getNodeById(id);
	g->addNode(n, false);
	/*
	fprintf(stderr, "\n\tcenter node(%s, %u) and types (%u) added", g->getNodeById(id)->getLabel(), id, g->getNumTypes());
	fflush(stderr);*/
	std::vector<const MyNode *> neighbors = this->getNeighborsToDegreeById(id, degree);
	//fprintf(stderr, "\n\tall %u-neighbors retrieved (%u)", degree, neighbors.size());
	//fflush(stderr);
	for(unsigned int i = 0; i < neighbors.size(); ++i)
	{
		g->addNode(neighbors[i], true);
	}
	//fprintf(stderr, "\n\tall %u-neighbors added", degree);
	//fflush(stderr);
	if(centerIdegree)
	{
		//corrected at 10:30pm 7/20/11 
		//updating i-degree requires that i be {1,2,3}
		for(unsigned int i=1;i <= 3;i++)
		{
			//g->minimumUpdatePositions(id,i,1);
			
			//fprintf(stderr, "\n\t%u-degree of center calculated", i);
			//fflush(stderr);
		}
	}
	else
	{
		g->updatePositions(1,args);
	}

	return g;
}


//gets subgraph by increasing the distance around id
MyGraph* MyGraph::getSubgraphOfSize(unsigned int id, unsigned int minSize,const arguments& args)
{
	std::vector<const MyNode *> neighbors;
	unsigned int dist = 0; 
	do{
		neighbors = this->getNeighborsToDegreeById(id, dist);
		dist++;
	}while(neighbors.size() < minSize -1);

	MyGraph * g = new MyGraph(this->isDirected());
	g->addNode(this->getNodeById(id), false);

	for(unsigned int i = 0; i < neighbors.size(); ++i)
	{
		g->addNode(neighbors[i], true);
	}

	g->updatePositions(1,args);
	return g;
}


//sets the 1, 2, 3 degrees of all nodes within the graph based on infinite locality
void MyGraph::updatePositions(int method,const arguments& args)
{
	/*
	fprintf(stderr, "\nTypes in graph (%u)", (unsigned)this->types.size());
	for(unsigned int j = 0; j < this->types.size(); ++j)
	{
		fprintf(stderr, "\n\ttype=%u, normalized=%u", this->types[j], this->normalizedType(this->types[j]) );
	}*/
	timing T;
	fprintf(stderr, "args.i_degree = %d\n",args.i_degree);
	//T.start("calcIDegree");
	for(int i = 1; i <= args.i_degree; ++i)
	{
		fprintf(stderr, "begin updating %u-degrees\n\n", i);
		//fflush(stderr);
		//unsigned int count = 0;
		for(mapIDNodes::iterator it = this->nodes.begin(); 
			it != this->nodes.end(); ++it)
		{
			this->calculateIDegree(it->second.getID(), method, i);
			//fprintf(stderr, "\n%u-deg(%u)", i, it->second->getID());
			/*if(count%1000==0)
			{T.stop("calcIDegree");
			T.print();
				fprintf(stderr, "\tupdated %d node %s (%u) neighbors(%u)\n", count, it->second->getLabel(), it->second->getID(), it->second->getNumAllNeighbors());
				fflush(stderr);
				T.start("calcIDegree");
			}
			count++;*/
		}
		fprintf(stderr, "finished updating %u-degrees\n", i);
		//fflush(stderr);
	}
}

std::vector<const MyNode *> MyGraph::findNodesOfType(unsigned short type) const
{
	std::vector<const MyNode *> nodesOfType;

	unsigned int start=0, end=this->nodeSorted.size();
	for(size_t k = 0; k < this->partitionStarts[0].size(); ++k)
	{
		if(this->nodeSorted[this->partitionStarts[0][k]]->getType() == type)
		{
			start = this->partitionStarts[0][k];
			if(k <this->partitionStarts[0].size() -1)
			{
				end = this->partitionStarts[0][k+1];
			}
			break;
		}
	}

	nodesOfType.reserve(end-start);
	//nodesOfType.assign(nodeSorted.begin()+start,nodeSorted.begin()+end);//TODO: double check this line and use it instead
	for(unsigned int i = start; i < end; ++i)
	{
		nodesOfType.push_back(this->nodeSorted[i]);
	}

	return nodesOfType;
}

std::vector<unsigned int> MyGraph::getSingletonIDs() const
{
	std::vector<unsigned int> singles;
	singles.reserve(100);
	for(mapIDNodes::const_iterator it = this->nodes.begin(); it != this->nodes.end(); ++it)
	{
		if(it->second.getNumAllNeighbors() == 0)
		{
			singles.push_back(it->second.getID());
		}
	}

	return singles;
}

/*** Mutators ***/

//adds a node of <type> with <label> to the graph, the ID of the node will be set to max_id+1 
//max_id and numNodes will be increased; uptodatePartitions will be set to false
//if successfully added a 1 will be returned else a 0 will be returned
unsigned int MyGraph::addNode(unsigned char llen, char* label,  unsigned short type)
{
	max_id++;
	return createNode(llen,label,type,max_id);
}

//adds a node of <type> with  <label> (converted to c-string) to the graph, the ID of the node will be set to either id or max_id+1 
//max_id and numNodes will be increased; uptodatePartitions will be set to false
//if successfully added a 1 will be returned else a 0 will be returned
unsigned int MyGraph::addNode(std::string label, unsigned short type, long int id)
{
	char* temp=stringToCString(truncateString(label));

	if(id==-1)
	{
		max_id++; 
		id=max_id;
	}

	return createNode(label.size(),temp,type,id);
}

//adds a node of <type> with  <label>  to the graph, with id: 
//max_id may be increased and numNodes will be increased; uptodatePartitions will be set to false
//if successfully added a 1 will be returned else a 0 will be returned
unsigned int MyGraph::addNode(unsigned int id, unsigned char llen, char* label, unsigned short type)
{
	if(id > this->max_id)
	{
		this->max_id = id;
	}

	return createNode(llen,label,type,id);
}

//adds an exact copy of n to the graph (the new node will have the same ID, type, and label. 
//if edges = true any incoming and outgoing edges of n with both endpoints in the graph, will be added
//max_id may be increased; num_nodes will be increased and uptodatePartions will be set to false; uptodateIndices &&= !edges
//if successfully added a 1 will be returned else a 0 will be returned
unsigned int MyGraph::addNode(const MyNode *n, bool edges)
{
	if(n == NULL)
	{
		return 0;
	}
	unsigned int result = addNode(n->getID(), n->getLabelLen(), n->getLabel(), n->getType());
	if (result == 0) 
	{
		return 0;
	}
	if (edges) {
		const edgelist* outedges=n->getOutEdgesConst();
		for (edgelist::const_iterator i=outedges->begin(); i != outedges->end(); i++) 
		{
			addEdge(n->getID(), i->first, i->second.getEdgeLabel());
		}
		if(directed)
		{
			const edgelist* inedges=n->getInEdgesConst();
			for (edgelist::const_iterator i=inedges->begin(); i !=inedges->end(); i++) 
			{
				addEdge(i->first, n->getID(),i->second.getEdgeLabel());
			}
		}
	}
	return result;
}//addNode


bool MyGraph::isEdge(unsigned int id1, unsigned int id2) const
{
	mapIDNodes::const_iterator left = nodes.find(id1), 
		right = nodes.find(id2);
	bool result = ( id1==id2 );

	if(!result && left != nodes.end() && right != nodes.end())
	{
		const edgelist* outedges=left->second.getOutEdgesConst();
		if(outedges->find(id2)!=outedges->end())
		{return true;}
	}

	return false;
}

//attemps to add an edge from id1 to id2 calls addDirectedEdge or addUndirectedEdge as needed
//returns success
bool MyGraph::addEdge(unsigned int id1, unsigned int id2,std::string label)
{
	bool result = id1!=id2 && nodes.find(id1) != nodes.end() && nodes.find(id2) != nodes.end() && !isEdge(id1, id2) ;
	if(result)
	{
		if(directed)
		{
			result = addDirectedEdge(id1, id2,label); 
		}
		else
		{
			result = addUndirectedEdge(id1, id2,label);
		}
	}
//don't print message for self loop or for undirected multiple edges
	bool verbose=false;
	if(result==false && verbose)
	{
		if( id1!=id2 || !(!this->isDirected() && this->isEdge(id1, id2) ))
		{
			fprintf(stderr, "\nError: edge(%u, %u) failed to add", id1, id2);
			if(isEdge(id1, id2) )
			{
				fprintf(stderr, " already an edge");
			}
			if(this->nodes.find(id1) == this->nodes.end() )
			{
				fprintf(stderr, " first id not in graph");
			}
			if(this->nodes.find(id2) == this->nodes.end())
			{
				fprintf(stderr, " second id not in graph");
			}
			fflush(stderr);
		}
	}
	if(result)
	{
		num_edges++;
	}
	return result;
}

//attempts to add a directed edge from ID1 to ID2: 
//if id1 == id2 returns false and doesn't add edge
//if an edge doesn't exist, id2 is added to id1.outgoing_edges and id1 is added to id2.incoming_edges
bool MyGraph::addDirectedEdge  (unsigned int id1, unsigned int id2,std::string label)
{
	bool result = getNodeById(id1)->addOutEdge(MyEdge(getNodeById(id2),label));
	if(getNodeById(id2)->addInEdge(MyEdge(getNodeById(id1),label)))
	{
		result=true;
	}
	//fprintf(stderr, "\n\tpushed onto id2 incoming");
	//boost::add_edge(getIndexById(id1),getIndexById(id2),*bGraph);
	return result;
}

//attempts to add an undirected edge between ID1 and ID2: 
//if id1 == id2 returns false and doesn't add edge
//if an edge doesn't exist, id2 is added to id1.outgoing_edges and id1 is added to id2.outgoing_edges
bool MyGraph::addUndirectedEdge(unsigned int id1, unsigned int id2,std::string label)
{
	bool result = nodes[id1].addOutEdge(MyEdge(&nodes[id2],label));
	//fprintf(stderr, "\n\tpushed onto id1 outgoing");
	if(nodes[id2].addOutEdge(MyEdge(&nodes[id1],label)))
	{
		result=true;
	}
	//fprintf(stderr, "\n\tpushed onto id2 outgoing");
	//boost::add_edge(getIndexById(id1),getIndexById(id2),*bGraph);
	return result;
}
//if force = false: removes a node only if it has no edges  uptodatePartitions is set to false
//if force = true: removes all Edges connected the node then removes the node, uptodatePartitions is set to false, uptodateIndices is set within removeEdges
bool MyGraph::removeNode(unsigned int id, bool force)
{
	bool removed = 0;
	mapIDNodes::iterator n = this->nodes.find(id);
	if(n == this->nodes.end()){
		return 0;
	}

	if(force){
		n->second.removeAllEdges();
	}

	if(n->second.getNumAllNeighbors() == 0)//not forced can't have edges
	{
		this->nodes.erase(n);//remove the id from the graph
		this->uptodatePartitions = false;
		removed = true;
	}
	else
	{
		fprintf(stderr,"Could not delete node %d, error.\n",n->second.getID());
	}

	return removed;
}
//removes all nodes with 0 incoming and outgoing edges.
unsigned int MyGraph::removeSingletonNodes()
{
	unsigned int initSize = this->getNumNodes();
	std::vector<unsigned int> singles = this->getSingletonIDs();

	for(unsigned int i = 0; i < singles.size(); ++i)
	{
		this->removeNode(singles[i], false);
	}

	if(this->getNumNodes() + singles.size() != initSize)
	{
		fprintf(stderr, "\n\tERROR: current size(%u) + singles(%u) != initial size (%u)", this->getNumNodes(), (unsigned)singles.size(), initSize);
		fflush(stderr);
	}

	fprintf(stdout, "\nAfter removing singletons: size %u", this->getNumNodes() );
	return 0;
}
//removes any and all edges from id1 to id2, sets uptodate Indices to false
//if the edge doesn't exist, returns 0 else returns 1
unsigned int MyGraph::removeEdge(unsigned int id1, unsigned int id2)
{
	unsigned int removed = 0;

	MyNode * left = getNodeById(id1); 
	MyNode * right = getNodeById(id2);

	std::vector<MyNode*>::iterator i, j;
	removed+=left->removeOutEdge(right);
	removed+=right->removeOutEdge(left);

	this->num_edges -= removed;

	if(removed > 0)
	{
		this->uptodateIndices = false;
		this->uptodatePartitions = false;
	}

	return removed;
}

//checks all outgoing/incoming edges of all nodes to ensure that for any node i with outgoing edges j that for each j,
//i is in the incoming of j.  for any which do not have valid pairs, the value is removed
unsigned int MyGraph::removeRogueEdges()
{
	fprintf(stdout,"RemoveRogueEdges is called\n");
	unsigned int removed = 0;

	for(mapIDNodes::iterator it = this->nodes.begin();
		it != this->nodes.end(); ++it)
	{
		//check outgoing
		edgelist* outedges=it->second.getOutEdges();
		for(edgelist::iterator i = outedges->begin(); 
			i != outedges->end(); i++)
		{
			mapIDNodes::iterator end = this->nodes.find(i->first);
			if(end == this->nodes.end() )
			{
				it->second.removeOutEdge((*i).second.getNode());
				++removed;
				continue;
			}

			bool found = false;
			if(this->directed)
			{
				const edgelist* inedges=end->second.getInEdgesConst();
				found=(inedges->find(it->first)!=inedges->end());
			} 
			else
			{
				edgelist* outedges2=end->second.getOutEdges();
				found=(outedges2->find(it->first)!=outedges2->end());
			}

			if(!found)
			{
				it->second.removeOutEdge((*i).second.getNode());
				++removed;
				continue;
			}
		}//for the outgoing edges

		const edgelist* inedges=it->second.getInEdgesConst();
		for(edgelist::const_iterator i = inedges->begin(); 
			i != inedges->end(); ++i)
		{
			mapIDNodes::iterator end = this->nodes.find(i->first);
			if(end == this->nodes.end() )
			{
				it->second.removeInEdge(i->second.getNode());
				++removed;
				continue;
			}

			bool found = false;
			edgelist* outedges=end->second.getOutEdges();
			found=outedges->find(it->first)!=outedges->end();

			if(!found)
			{
				it->second.removeInEdge(i->second.getNode());
				++removed;
				continue;
			}
		}
	}//for each node

	return removed;
}


/* Core Functionality */

//generates a graph randomly using a normal distribution for determining the number of edges per node
//
unsigned long MyGraph::generateGraph(unsigned int numnodes, unsigned short num_types, unsigned short edges_per_node,const arguments& args,bool directed)
{
	this->directed=directed;
	for(unsigned int i=0;i<numnodes;i++)
	{
		addNode(8,randomLabel(8),rand()%num_types);
	}
	unsigned num_edges,id;
	boost::mt19937 rng;
	for(mapIDNodes::iterator itr=nodes.begin();itr!=nodes.end();itr++)
	{
		num_edges=(unsigned int)((edges_per_node)*(normalDistribution(rng)+1.0));
		for(unsigned int i=0;i<num_edges;i++)
		{
			do
			{
				id=rand()%numnodes;
			}while(id==itr->second.getID());
		
			addEdge(itr->second.getID(),id,"*");
		}
	}
	this->updatePositions(1,args);


	return 0;
}



/* methods for finding fragments */
// USE Similarity() in utilities.cpp float Graph::distanceBetweenNodes(char similarityMethod,const Node* center, const Node* center2, unsigned int i_degree)
//{
//	float d=-1.0;
//	switch(similarityMethod)
//	{
//	case 'd':
//		d = dist(center, center2, i_degree); 
//		break;
//	case 'c':
//		d = coverDist(center, center2, i_degree);
//		break;
//	case 'n':
//		d = nonZeroDist(center, center2, i_degree);
//		break;
//	case 'w':
//		d=weightDist(center, center2, i_degree);
//		break;
//	case 'r':
//		d = rand();
//		break;
//	default:
//		printf("\nUnknown distance measure %c", similarityMethod);
//		break;
//	}
//	return d;
//}
//uses i-degree and distance to find candidates for some node
std::vector<std::pair<double,unsigned int> > MyGraph::find_by_i(const MyNode * center, char similarityMethod, unsigned int i_degree, unsigned int nBest, char distMethod,struct arguments args)
{
	std::vector<std::pair<double, unsigned int> > candidates;
	double d;

	candidates.reserve(nBest);

	unsigned int startPartition=0, endPartition=nodeSorted.size();
	getStartEndPartition(startPartition,endPartition,center);

	for(unsigned int i = startPartition; i < endPartition; ++i)
	{
		if(nodeSorted[i]->getType() != center->getType() || nodeSorted[i]->getNumOutNeighbors() <= FEW_EDGES || nodeSorted[i]->getNumOutNeighbors() >= MANY_EDGES)
		{
			continue;
		}
		d=similarity(this->nodeSorted[i], center, similarityMethod, i_degree, distMethod,args);
		if(d < 0)
		{
			continue;
		}
		updateBestCandidates(candidates, std::pair<double,unsigned int>(d,nodeSorted[i]->getID()), nBest);
	}
	
	return candidates;
}

void MyGraph::getStartEndPartition(unsigned int &start,unsigned int &end,const MyNode* center)const
{
	start=0;
	end=nodeSorted.size();
	for(unsigned int i = 0; i < partitionStarts[0].size(); ++i)
	{
		if(nodeSorted[partitionStarts[0][i]]->getType() == center->getType())
		{
			start = partitionStarts[0][i];
			if(i < partitionStarts[0].size()-1)
			{
				end = partitionStarts[0][i+1];
			}
		}
	}
}
#define TRACE_find_by
std::vector<std::pair<double,unsigned int> > MyGraph::find_by_tree(const MyNode* center,const MyGraph* fragment, char similarityMethod, unsigned int i_degree, unsigned int nBest, unsigned int N, char distMethod, struct arguments args)
{
#ifdef TRACE_find_by
	fprintf(stderr, "\nIn graph::find_by_tree");
#endif
	if(center->getNumOutNeighbors()< N)
	{
		fprintf(stderr,"Not enough neighbors(%u) for %d-tree search: %d\n",center->getNumOutNeighbors(),N,center->getID());
		std::vector<std::pair<double,unsigned int> > reduce(find_by_tree(center,fragment,similarityMethod,i_degree,nBest,center->getNumOutNeighbors(),distMethod,args));
		return reduce;
	}
	std::vector<unsigned int> best_neighbors;//=find_N_best_neighbors(center, 'd', N);
	std::vector<std::vector<std::pair<double,unsigned int> > > tempCandidates(N,std::vector<std::pair<double,unsigned int> >(N,std::pair<double,unsigned int>(-1,0)));
	std::vector<std::pair<double,unsigned int> > candidates;
	candidates.reserve(nBest);
	unsigned int i_less1 = (i_degree<=1?i_degree:i_degree -1);
	unsigned int startPartition,endPartition;
	getStartEndPartition(startPartition,endPartition,center);
	
	for(unsigned int i = startPartition; i < endPartition; ++i)
	{
		//check if this node meets type and min edge requirements
		if(nodeSorted[i]->getType() != center->getType() || nodeSorted[i]->getNumOutNeighbors() <= FEW_EDGES || nodeSorted[i]->getNumOutNeighbors() >= MANY_EDGES)
		{
			continue;
		}

		//check if this node has at least one adj node similar to neighbor
		std::vector<std::pair<const MyNode*, double> > neighbors(N, std::pair<const MyNode*, double>((const MyNode*)NULL, 0.0));
		bool found=true;
		std::vector<const MyNode *> limitations;
		limitations.push_back(center);
		for(unsigned int j=0;j<N&&found;j++)
		{
			for(unsigned int k=0;k<N&&found;k++)
			{
				//this version of this function is being removed. //neighbors[k]=this->findBestMatchingNeighbor(nodeSorted[i],fragment->getNodeByIdConst(best_neighbors[j]),limitations,similarityMethod,i_less1,distMethod,args);
				if(neighbors[k].first==NULL)
				{
					if(k==0){
						continue;}
					break;
				}
				limitations.push_back(neighbors[k].first);
				tempCandidates[j][k]=std::pair<double,unsigned int>(neighbors[k].second,neighbors[k].first->getID());
				if(tempCandidates[j][0].second<0.0)
				{
					found=false;
					break;
				}
			}
		}
		if(!found)
		{
			continue;
		}
		double d = similarity(this->nodeSorted[i], center,similarityMethod, i_degree,distMethod,args);
		if(d < 0.0)
		{
			continue;
		}
		for(unsigned int j=0;j<N;j++)
		{
			d+=tempCandidates[j][0].first;
		}
		updateBestCandidates(candidates, std::pair<double,unsigned int>(d,nodeSorted[i]->getID()), nBest);
	}

	return candidates;
}

float MyGraph::NtreeSimilarity(unsigned int N, const MyNode *center, const MyGraph* fragment, MyNode* candidate, unsigned int i_degree, char similarityMethod, std::vector<unsigned int> best_neighbors, char distMethod,struct arguments args)
{
	float d=0,tempD=0;
	d = similarity(candidate, center, similarityMethod, i_degree,distMethod, args);
	if(d<0.0)
	{
		return -1;
	}
	//fprintf(stderr,"begin ntree\n");
	std::vector<const MyNode *> limitations;
	limitations.push_back(center);
	//check if this node has at least one adj node similar to neighbor
	unsigned int i_less1 = (i_degree<=1?i_degree:i_degree -1);
	
	for(unsigned int j=0;j<N;j++)
	{
		const MyNode* neighbor=fragment->getNodeByIdConst(best_neighbors[j]);
		
		std::pair<const MyNode *, float> bestNeighbor ;//this->findBestMatchingNeighbor(candidate, neighbor, limitations, similarityMethod, i_less1,distMethod,args);
		limitations.push_back(bestNeighbor.first);
		if(bestNeighbor.first==NULL)
		{
			return -1;
		}
		tempD = bestNeighbor.second;

		//if no valid match for neighbor existed, move on to next potential
		if(tempD < 0.0)
		{
			return -1;
		}
		d+=tempD;
	}
	return d;
}

std::vector<std::pair<double,std::map<unsigned int, unsigned int> > > MyGraph::find_by_NtreeNeighborhood(const MyNode *center, const MyGraph * fragment, char similarityMethod, unsigned int i_degree, unsigned int nBest,unsigned int N, char distMethod,struct arguments args)
{
	fprintf(stderr, "\nIn graph::find_by_Ntree(%u, fragment, %c, %u, %u, %u)", center->getID(), similarityMethod, i_degree, nBest, N);
	unsigned int n = center->getNumOutNeighbors();
	if(n < N)
	{
		fprintf(stderr,"\nNot enough neighbors(%u) for %d-tree search: %d\n",center->getNumOutNeighbors(),N,center->getID());

		std::vector<std::pair<double,std::map<unsigned int, unsigned int> > > reduce;//(find_by_NtreeNeighborhood(center,fragment,similarityMethod,i_degree,nBest,n, distMethod,args));
		return reduce;
	}
	std::vector<unsigned int> best_neighbors;//=fragment->find_N_best_neighbors(center, 'd', N);
	fprintf(stderr, "\n\tgot best neighbors");
	
	std::vector<std::pair<double,std::map<unsigned int, unsigned int> > > candidates;
	
	unsigned int startPartition=0, endPartition=nodeSorted.size();
	getStartEndPartition(startPartition,endPartition,center);
	fprintf(stderr, "\n\tgot partition (%u, %u)\n", startPartition, endPartition);
	unsigned int reserveSize=(((endPartition-startPartition)>=nBest)? nBest : (endPartition-startPartition));
	candidates.reserve(reserveSize);

	for(unsigned int i = startPartition; i < endPartition; ++i)
	{
		//check if this node meets type and min edge requirements
		if(nodeSorted[i]->getType() != center->getType() || nodeSorted[i]->getNumOutNeighbors() <= FEW_EDGES || nodeSorted[i]->getNumOutNeighbors() >= MANY_EDGES)
		{
			continue;
		}
		std::pair<double,std::map<unsigned int, unsigned int> > d;
		if(similarityMethod=='e'||similarityMethod=='E')//use old, "Ntree" method of greedy mapping.
		{
			fprintf(stderr,"Depricated 'e' or 'E' argument\n");
			exit(0);
			//d=NtreeSimilarity(N, center, fragment, this->nodeSorted[i], i_degree, similarityMethod+1, best_neighbors,distMethod,args);
		}
		else
		{
			d;//=bGraphMappingNeighborhood( center, fragment, this->nodeSorted[i], i_degree, similarityMethod, best_neighbors,distMethod,args);
		}
		if(d.first<0.0)
		{
			continue;
		}
		//fprintf(stderr,"updateCands %f\n",d);
		//add to best, and sort only if there are nBest<=all nodes in the partition
		updateBestCandidates(candidates, d, nBest,(nBest<=endPartition-startPartition));
		//fprintf(stderr,"candidates updated...next\n");
		
	}

	fprintf(stderr, "\n\tfound %u candidates", (unsigned int) candidates.size());
	fprintf(stdout, "\n\tfound %u candidates", (unsigned int) candidates.size());
	if(nBest > endPartition-startPartition)
	{
		sort(candidates.begin(), candidates.end());
	}
	return candidates;
}



#define TRACE_find_by
std::vector<std::pair<double,unsigned int> > MyGraph::find_by_Ntree(const MyNode *center, const MyGraph * fragment, char similarityMethod, unsigned int i_degree, unsigned int nBest,unsigned int N, char distMethod,struct arguments args)
{
	fprintf(stderr, "\nIn graph::find_by_Ntree(%u, fragment, %c, %u, %u, %u)", center->getID(), similarityMethod, i_degree, nBest, N);
	unsigned int n = center->getNumOutNeighbors();
	if(n < N)
	{
		fprintf(stderr,"\nNot enough neighbors(%u) for %d-tree search: %d\n",center->getNumOutNeighbors(),N,center->getID());

		std::vector<std::pair<double,unsigned int> > reduce(find_by_Ntree(center,fragment,similarityMethod,i_degree,nBest,n, distMethod,args));
		return reduce;
	}
	std::vector<unsigned int> best_neighbors;//=fragment->find_N_best_neighbors(center, 'd', N);
	fprintf(stderr, "\n\tgot best neighbors");
	
	std::vector<std::pair<double,unsigned int> > candidates;
	
	unsigned int startPartition=0, endPartition=nodeSorted.size();
	getStartEndPartition(startPartition,endPartition,center);
	fprintf(stderr, "\n\tgot partition (%u, %u)\n", startPartition, endPartition);
	unsigned int reserveSize=(((endPartition-startPartition)>=nBest)? nBest : (endPartition-startPartition));
	candidates.reserve(reserveSize);

	for(unsigned int i = startPartition; i < endPartition; ++i)
	{
		//check if this node meets type and min edge requirements
		if(nodeSorted[i]->getType() != center->getType() || nodeSorted[i]->getNumOutNeighbors() <= FEW_EDGES || nodeSorted[i]->getNumOutNeighbors() >= MANY_EDGES)
		{
			continue;
		}
		double d=0.0;
		if(similarityMethod=='e'||similarityMethod=='E')//use old, "Ntree" method of greedy mapping.
		{
			d=NtreeSimilarity(N, center, fragment, this->nodeSorted[i], i_degree, similarityMethod+1, best_neighbors,distMethod,args);
		}
		else
		{
			d;//=bGraphMapping( center, fragment, this->nodeSorted[i], i_degree, similarityMethod, best_neighbors,distMethod,args);
		}
		if(d<0.0)
		{
			continue;
		}
		//fprintf(stderr,"updateCands %f\n",d);
		//add to best, and sort only if there are nBest<=all nodes in the partition
		updateBestCandidates(candidates, std::pair<double,unsigned int>(d,nodeSorted[i]->getID()), nBest,(nBest<=endPartition-startPartition));
		//fprintf(stderr,"candidates updated...next\n");
		
	}

	fprintf(stderr, "\n\tfound %u candidates", (unsigned int) candidates.size());
	fprintf(stdout, "\n\tfound %u candidates", (unsigned int) candidates.size());
	if(nBest > endPartition-startPartition)
	{
		sort(candidates.begin(), candidates.end());
	}
	return candidates;
}


std::vector<std::pair<double,std::map<unsigned int, unsigned int> > > MyGraph::find_by_itPart(const MyNode *center, const MyGraph* fragment, struct arguments args)
{
	bagsType bags;
	std::vector<MyNode*>::const_iterator X;
	for(X=fragment->nodeSorted.begin();X!=fragment->nodeSorted.end();X++)
	{//for each node X, in the fragment F
		for(std::vector<unsigned long long int>::iterator itr=itPart.rootPtr->subpartitions.begin();
			itr!=itPart.rootPtr->subpartitions.end(); itr++)
		{//for each subpartition of root
			if(itPart.getPartition(*itr)->nodes[0]->getType()==(*X)->getType())
			{//if the node in this subpartition is the same type as X
				bags[(*X)->getID()][(*itr)]=0;
				//then initialize Q(X) to the partition of nodes with the same type as X.
			}
		}
	}
	bagsType bags2;
	
	//speed improvement: use 1 degree if this is a type partition
	for(bagsType::iterator bItr=bags.begin();bItr!=bags.end();bItr++)
	{
		for(mappingType::iterator mItr=bItr->second.begin();mItr!=bItr->second.end();mItr++)
		{
			for(std::vector<unsigned long long int>::iterator cItr=itPart.getPartition(mItr->first)->subpartitions.begin();cItr!=itPart.getPartition(mItr->first)->subpartitions.end();cItr++)
			{
				if(similarity(itPart.getPartition(*cItr)->nodes[0],fragment->getNodeByIdConst(bItr->first), 'F', 1, '1',args)!=-1)
				{
					bags2[bItr->first][*cItr]=0;
				}
			}
		}
	}
	return find_by_itPart_iterate(center,fragment,args,bags2,itPart,0);
}
std::vector<std::pair<double,std::map<unsigned int, unsigned int> > > MyGraph::find_by_itPart_iterate(const MyNode *center, const MyGraph* fragment, struct arguments args, bagsType& bags,MyPartitioning iterPart,unsigned int depth)
{
	float dLimit=0.0;
	std::vector<std::pair<double,std::map<unsigned int,unsigned int> > > returnVal;
	std::map<MyNode*,bool> uniqueResults;
	std::map<unsigned int, bool> rechecker,nextChecker;
	std::vector<MyNode*>::const_iterator X;
	//init the bags
	bagsType bags2;//maps fragment node to a map of (candidates to penalties).
	for(X=fragment->nodeSorted.begin();X!=fragment->nodeSorted.end();X++)
	{//for each node X, in the fragment F
		rechecker[(*X)->getID()]=true;
		nextChecker[(*X)->getID()]=true;
	}
	bool notChosenToExpand=true;
	unsigned int singles=0,divisor=0;
	float avgsingles=0;
	std::vector<std::pair<unsigned int,unsigned int> > tempOptions;
	MyPartitioning tempItPart;
	unsigned int removedOptions=0;
	long double totalEval=0;
	long double bestEval=0;
	long double originalEval=1;
#ifdef PARALLEL
	const unsigned int NUM_THREADS = 2;
	pthread_t handle[NUM_THREADS];
	pthread_mutex_t 	lock;
	pthread_mutex_t		lockID;
	pthread_mutex_t		lockCont;
	
	pthread_mutex_init( &lock, NULL);
	pthread_mutex_init( &lockID,NULL);
	pthread_mutex_init( &lockCont,NULL);
#else
	const unsigned int NUM_THREADS = 1;
#endif
	bool cont=true;
	unsigned int maxHeight=100;
	unsigned int c=0;
	
	//end speed improvement
	while(cont&&maxHeight!=0)
	{
		std::vector< bagsType > returnVal(NUM_THREADS);
		maxHeight=maxHeight-1;
		cont=false;
		findByItPartArgs u;
		{
			u.increment=NUM_THREADS;
			u.start=fragment->nodeSorted.begin();
			u.G=this;
			#ifdef PARALLEL
				u.lock=lock;
				u.lockID=lockID;
				u.lockCont=lockCont;
			#endif
			u.fragment=fragment;
			u.rechecker=&rechecker;
			u.nextChecker=&nextChecker;
			u.bags=bags;
			u.dLimit=dLimit;
			u.nextID=0;
			u.args=&args;
			u.returnVal=returnVal;
			u.cont=false;
			u.iterPart=&iterPart;
		}
		
#ifndef PARALLEL
		findByItPartThread(&u);
#else
		for(unsigned int thread=0;thread<NUM_THREADS;thread++)//unsure
		{
			int error=pthread_create(&handle[thread],NULL,findByItPartThread, &u);
			if(error)
			{
				fprintf(stderr,"\nError: failed to create thread %d, (total threads =%d)", thread, NUM_THREADS);
				exit(EXIT_FAILURE);
			}

		}
		for(unsigned int thread =0; thread <NUM_THREADS; thread++)
		{
			pthread_join(handle[thread], NULL);
		}
#endif
		//MERGE ANSWERS HERE
		cont=u.cont;
		for(unsigned int thread=0;thread<NUM_THREADS;thread++)
		{
			for(bagsType::iterator itr=u.returnVal[thread].begin();
				itr!=u.returnVal[thread].end();itr++)
			{
				if(bags2.find(itr->first)==bags2.end())
				{
					bags2[itr->first]=itr->second;
				}
				else
				{
					bags2[itr->first].insert(itr->second.begin(),itr->second.end());
				}
			}
		}
		//incomplete: remove unique node mappings from other nodes.
		for(bagsType::iterator bag=bags2.begin();
			bag!=bags2.end();bag++)
		{
			if(bag->second.size()==1 && iterPart.getPartition(bag->second.begin()->first)->nodes.size()==1)
			{
				//there is a unique bag
				for(bagsType::iterator bag2=bags2.begin(); bag2!=bags2.end();bag2++)
				{
					if(bag2!=bag)
					{
						if(bag2->second.find(bag->second.begin()->first)!=bag2->second.end())
						{
							bag2->second.erase(bag->second.begin()->first);
							nextChecker[bag2->first]=true;
						}
					}
				}
			}
		}
		c=0;
		for(bagsType::iterator bag=bags2.begin();
			bag!=bags2.end();bag++)
		{
			//fprintf(stdout,"bags= %d\n",bag->second.size());
			if(bag->second.size()==1)
			{	
				if(iterPart.getPartition(bag->second.begin()->first)->nodes.size()==1)
				{
					c++;
				}
			}
		}
		//fprintf(stdout,"singletons = %d\n",c);
		bags=bags2;//set Q(X)=Q'(X);
		bags2.clear();
		rechecker=nextChecker;
		nextChecker.clear();
		mappingType::iterator i;
		unsigned printVal=0;
		//fprintf(stdout,"Begin\n");
		for(i=bags[center->getID()].begin();i!=bags[center->getID()].end();i++)
		{
			printVal+=iterPart.getPartition(i->first)->nodes.size();
			//printDegree(i->first->nodes[0],stdout);
		}
		fprintf(stderr,"found %d\n",printVal);
		//fprintf(stdout,"found %d\n",printVal);
		//exit(0);
		
	}
	//before finishing, try to repartition
	//if(repartitions==0)
	//{
		//repartitions++;
		fprintf(stderr,"Options\n");
		//we want to output the number of options 
		//count number of options per node:
		for(bagsType::iterator itr=bags.begin();
			itr!=bags.end(); itr++)
		{//for each node in the fragment
			if(itr->second.size()==0)
			{
				fprintf(stderr,"No possibilities\n");
				return returnVal;//error, there needs to be possibilities.
			}
			unsigned int myTemp=0;
			//printDegree(fragment->getNodeById(itr->first),stderr);
			for(mappingType::iterator jtr=itr->second.begin();jtr!=itr->second.end();jtr++)
			{
				myTemp+=iterPart.getPartition(jtr->first)->nodes.size();
				//verbose
				/*for(std::vector<MyNode*>::iterator ktr=iterPart.getPartition(jtr->first)->nodes.begin(); ktr!=iterPart.getPartition(jtr->first)->nodes.end();ktr++)
				{
					fprintf(stderr,"%d->%d\n",itr->first,(*ktr)->getID());
					printDegree(*ktr,stderr);
				}*/
				//end verbose
			}
			tempOptions.push_back(std::pair<unsigned int, unsigned int>(myTemp,itr->first));
		}
		sort(tempOptions.begin(),tempOptions.end());
		
	mappingType::iterator yOption;
	unsigned int yExpand=0,yCount=0,yNodeNum=0;
		//new method
		fprintf(stderr,"|M(x)|- \n");
		for(unsigned int i=0;i<tempOptions.size();i++)
		{
			//this if statement finds the first option to explore
			//don't delete it!
			if(notChosenToExpand==true && tempOptions[i].first>1)
			{
				notChosenToExpand=false;
				yExpand=i;
				yOption=bags[tempOptions[yExpand].second].begin();
				fprintf(stderr,"Chosen\n");
			}
			fprintf(stderr,"%d - %d\n",tempOptions[i].first, tempOptions[i].second);//#3
			//fprintf(stderr,"originalEval = %Lf\n",originalEval);
			originalEval*=tempOptions[i].first;
		}
		fprintf(stdout,"originalEval = %Lf\n",originalEval);
		singles=c;
		
	//choose pivot:
	if(depth>0)
	{
		unsigned int tempYExpand=yExpand;
		std::list<std::map<unsigned int,unsigned int> > tempMaps;
		const unsigned int minBranchNodes=5;
		const unsigned int branchNodes=5;//arbitrary
		if(tempYExpand<minBranchNodes)
		{
		for(unsigned int pivotIterator=yExpand+1;pivotIterator<yExpand+branchNodes && pivotIterator<tempOptions.size();pivotIterator++)
		{
			std::list<std::map<unsigned int,unsigned int> > tempMaps2;
			tempMaps.insert(tempMaps.end(),tempMaps2.begin(),tempMaps2.end());
		}
		}
		compareItPartMappings(tempMaps);
		//this assumes depth1 TODO change this
		
	}
	else
	{
		fprintf(stderr,"returnVal.size() = %d\n",returnVal.size());
		
		//std::map<unsigned int, unsigned int> tempX;
		//for(bagsType::iterator itr=bags.begin();
		//	itr!=bags.end(); itr++)
		//{//for each node in the fragment
		//	if(itr->second.size()==0)
		//	{
		//		fprintf(stderr,"No possibilities\n");
		//		return returnVal;//error, there needs to be possibilities.
		//	}
		//	mappingType::iterator jtr=itr->second.begin();
		//	//fprintf(stderr,"%d\n",fragment->getNodeById(itr->first)->getID());
		//	tempX[fragment->getNodeById(itr->first)->getID()]=iterPart.getPartition(jtr->first)->nodes[0]->getID();
		//}
		//returnVal.push_back(std::pair<float, std::map<unsigned int, unsigned int> >(0,tempX));

		//std::vector<std::pair<double,unsigned int> > tempResult= 
		returnVal= calculateOptions(fragment,bags,center,args,iterPart);	
	
		fprintf(stdout,"found %d\n",(unsigned)returnVal.size());
		

		return returnVal;
	}
	fprintf(stderr,"-returnVal()= %d\n",returnVal.size());
	
	
	
return returnVal;
}
void* findByItPartThread(void* u)
{
	findByItPartArgs* args=(findByItPartArgs*)u;
	unsigned ID=0;
	//getID
	#ifdef PARALLEL
	{
		pthread_mutex_lock(&(args->lockID));
		ID=args->nextID;
		args->nextID++;
		pthread_mutex_unlock(&(args->lockID));
	}
	#endif
	bool cont=false;
	for(std::vector<MyNode*>::const_iterator X=args->start;X!=args->fragment->nodeSorted.end();X++)
	{//for each node X in the fragment F
		
		if(args->rechecker->find((*X)->getID())==args->rechecker->end())//if don't recheck
		{
			for(mappingType::iterator Qx=args->bags[(*X)->getID()].begin();
			Qx!=args->bags[(*X)->getID()].end();Qx++)
			{//for each partition that X can be mapped to, Q(X)
				args->returnVal[ID][(*X)->getID()][Qx->first]=0;//value;
			}
			continue;
		}
		//fprintf(stderr,"-%d\n",bags[(*X)->getID()].size());
		bool firstItr=true;
		for(mappingType::iterator Qx=args->bags[(*X)->getID()].begin();
			Qx!=args->bags[(*X)->getID()].end();)
		{//for each partition that X can be mapped to, Q(X)
			if(firstItr)
			{
				for(unsigned temp=0;temp<ID;temp++)
				{
					Qx++;
				}
				firstItr=false;
			}
			else
			{
				for(unsigned temp=0;Qx!=args->bags[(*X)->getID()].end() && temp<args->increment;temp++)
				{
					Qx++;
				}
			}
			if(Qx==args->bags[(*X)->getID()].end())
			{
				break;
			}
			
		
		//fprintf(stderr,"--%d\n",Qx->first->subpartitions.size());
			if(args->iterPart->getPartition(Qx->first)->subpartitions.size()==0)
			{
				unsigned int value;//=args->G->itPartMap((*X),args->fragment,args->iterPart->getPartition(Qx->first),args->bags,*(args->args),args->dLimit,args->iterPart);
				if(value>0)
				{//if this map is inviable
					//don't add it to Q'(X)
					cont=true;
					edgelist* outedges=(*X)->getOutEdges();
					edgelist* inedges=(*X)->getInEdges();
					for(edgelist::iterator o=outedges->begin();
						o!=outedges->end(); o++)
					{
						(*(args->nextChecker))[o->first]=true;
					}
					for(edgelist::iterator o=inedges->begin();
						o!=inedges->end(); o++)
					{
						(*(args->nextChecker))[o->first]=true;
					}
				}
				else
				{//this map was good, add it to Q'(X)
					args->returnVal[ID][(*X)->getID()][Qx->first]=0;//value;
				}
				//bags2[(*X)->getID()][Qx->first]=0;//don't just add, recheck.
			}
			else
			{
				cont=true;
				edgelist* outedges=(*X)->getOutEdges();
				edgelist* inedges=(*X)->getInEdges();
				
				for(edgelist::iterator o=outedges->begin();
					o!=outedges->end(); o++)
				{
					(*(args->nextChecker))[o->first]=true;
				}
				for(edgelist::iterator o=inedges->begin();
					o!=inedges->end(); o++)
				{
					(*(args->nextChecker))[o->first]=true;
				}
				for(std::vector<unsigned long long int>::iterator y=args->iterPart->getPartition(Qx->first)->subpartitions.begin();
					y!=args->iterPart->getPartition(Qx->first)->subpartitions.end();y++)
				{//for each subpartition of Q(X), y
					//fprintf(stderr,"a\n");
					unsigned int value;//=args->G->itPartMap((*X),args->fragment,(args->iterPart->getPartition(*y)),args->bags,*(args->args),args->dLimit,args->iterPart);
					/*if(maxHeight<9 && value<=0 && similarity((*y)->nodes[0],(*X),'F', 1, '1', args)<0)
					{
						fprintf(stdout,"error! %s\n",Qx->first->num.c_str());
						printDegree((*X),stdout);
						printDegree((*y)->nodes[0],stdout);
						exit(0);
					}*/
					if(value>0)
					{//if this map is inviable
						//don't add it to Q'(X)
						//fprintf(stdout,"b\n");
					}
					else
					{//this map was good, add it to Q'(X)
						args->returnVal[ID][(*X)->getID()][(*y)]=0;//value;
						//fprintf(stdout,"B\n");
					}
				}
			}
		}
	}
	if(cont)
	{
		#ifdef PARALLEL
		//mutex continue
		pthread_mutex_lock(&(args->lockCont));
		args->cont=true;
		pthread_mutex_unlock(&(args->lockCont));
		//endmutex continue
		#else
		args->cont=true;
		#endif
	}
	return NULL;
}
std::vector<std::pair<double,std::map<unsigned int, unsigned int> > > MyGraph::calculateOptions(const MyGraph* fragment,bagsType bags, const MyNode* center,struct arguments args,MyPartitioning& iterPart)
{
	fprintf(stderr,"Options\n");
	std::vector<std::pair<double,std::map<unsigned int, unsigned int> > > returnVal;
			
	//we want to output the number of options 
	//count number of options per node:
	std::vector<std::pair<unsigned int,unsigned int> > tempOptions;
	std::map<unsigned int, unsigned int> optionCount;
	for(bagsType::iterator itr=bags.begin();
		itr!=bags.end(); itr++)
	{//for each node in the fragment
		optionCount[itr->first]=0;
		for(mappingType::iterator jtr=itr->second.begin();
			jtr!=itr->second.end(); jtr++)
		{//for each partition that node can be in

			optionCount[itr->first]+=iterPart.getPartition(jtr->first)->nodes.size();
		}
		tempOptions.push_back(std::pair<unsigned int, unsigned int>(optionCount[itr->first],itr->first));
		//fprintf(stderr,"Option: %d\n",optionCount[itr->first]);
		if(optionCount[itr->first]==0)
		{
			fprintf(stderr,"No possibilities- \n");
			return returnVal;
			//This map has no possibilities
			//this means that either the input doesn't exist
			//or there was a node which was forced to map incorrectly.
		}
	}
	sort(tempOptions.begin(),tempOptions.end());
	std::vector<unsigned int> order;
	
	//new method
	fprintf(stderr,"|M(x)|\n");
	for(unsigned int i=0;i<tempOptions.size();i++)
	{
		order.push_back(tempOptions[i].second);
		fprintf(stderr,"%d - %d\n",tempOptions[i].first, tempOptions[i].second);//#3
	}
	//std::list<std::map<unsigned int,unsigned int> > r=mapItPartBFS(fragment,bags,0,order,center->getID(),iterPart);
	//
	
	
//Fancy Addition
//this modifies the existing i-partitioning algorithm to add in i-degree restrictions.
//This method is ok, but a more perminant version will be implemented later.
std::map<unsigned int, std::vector<unsigned int> > restrictions;
/*
for(std::vector<MyNode*>::const_iterator nodeItr=fragment->nodeSorted.begin();nodeItr!=fragment->nodeSorted.end();nodeItr++)
{
std::vector<std::pair< double, std::map<unsigned int, unsigned int> > > tempR=find_by_NtreeNeighborhood(*nodeItr,fragment, args.similarityMethod, 3, 4000000,9999999,args.distMethod,args);
for(std::vector<std::pair< double, std::map<unsigned int, unsigned int> > >::iterator mappingItr=tempR.begin();mappingItr!=tempR.end();mappingItr++)
{
	restrictions[(*nodeItr)->getID()].push_back(mappingItr->second[(*nodeItr)->getID()]);
}
}*/
//end fancy addition

	fprintf(stderr,"Begin greedy mapping\n");
	//std::list<std::map<unsigned int, unsigned int> > r= mapItPartGreedy(fragment,bags,0,order,center->getID(),iterPart,args);
	std::list<std::map<unsigned int, unsigned int> >   r= mapItPartGreedy(fragment,bags,0,order,center->getID(),iterPart,args,restrictions);
	fprintf(stderr,"End Greedy Mapping\n");
	fprintf(stdout,"# maps = %u\n",(unsigned)r.size());
	for(std::list<std::map<unsigned int, unsigned int> >::iterator itr=r.begin();
		itr!=r.end();itr++)
	{
			double d=verifyEdges(fragment,(*itr),args);
			if(d<0)
			{
				fprintf(stderr,"Mapping Incorrect...\n");
				//continue;
			}
			
			std::pair<double, std::map<unsigned int, unsigned int> > p(d,(*itr));
			returnVal.push_back(p);
	}
	fprintf(stderr,"#returnVal= %d\n",returnVal.size());
	sort(returnVal.begin(), returnVal.end());
	if(returnVal.size()>0)
	{
		fprintf(stdout,"%f %f\n",returnVal[0].first,returnVal[returnVal.size()-1].first);
	}
	return returnVal;
}
double MyGraph::verifyEdges(const MyGraph* fragment, std::map<unsigned int, unsigned int> &mapping,struct arguments args)
{
	double d=0;
	//this checks dominating by the way
	for(std::map<unsigned int, unsigned int>::iterator m=mapping.begin();
				m!=mapping.end();m++)
	{
		//add score for this node
		//fprintf(stdout,"similarity check\n");
		MyNode* S=getNodeById(m->second);
		float temp=similarity(S,fragment->getNodeByIdConst(m->first), 'F', 1, '1', args);
		if(temp==-1)
		{
			return -1;
		}
		if(getNodeById(m->second)->getNumAllNeighbors()>0)//if there are edges
		{
			float denom=S->getNumOutNeighbors()+S->getNumInNeighbors();
			if(denom!=0)
			{
				float newtemp=temp/denom;
				d+=newtemp;
				//fprintf(stderr,"%f -> %f\n",temp, newtemp);
			}
			
		}
		//end
		for(std::map<unsigned int, unsigned int>::iterator n=mapping.begin();
				n!=mapping.end();n++)
		{
			if(m->first==n->first)
			{
				continue;
			}
			bool fragmentEdge=fragment->isEdge(m->first,n->first);
			bool databaseEdge=isEdge(m->second,n->second);
			if(fragmentEdge && ! databaseEdge)
			{
				return -1;
			}
			if(!fragmentEdge && databaseEdge )
			{
				d+=1.0;
			}
		}
	}
	return d;
}

double MyGraph::verifyEdges(const MyGraph* fragment, std::map<unsigned int, unsigned int> &mapping,struct arguments args,MyNode* question)//check only the question node
{
	double d=0;
	//this checks dominating by the way
	for(std::map<unsigned int, unsigned int>::iterator m=mapping.begin();
				m!=mapping.end();m++)
	{
		//add score for this node
		//fprintf(stdout,"similarity check\n");
		MyNode* S=getNodeById(m->second);
		float temp=similarity(S,fragment->getNodeByIdConst(m->first), 'F', 1, '1', args);
		if(temp==-1)
		{
			return -1;
		}
		if(getNodeById(m->second)->getNumAllNeighbors()>0)//if there are edges
		{
			float denom=S->getNumOutNeighbors()+S->getNumInNeighbors();
			if(denom!=0)
			{
				float newtemp=temp/denom;
				d+=newtemp;
				//fprintf(stderr,"%f -> %f\n",temp, newtemp);
			}
			
		}
		//end
		std::map<unsigned int, unsigned int>::iterator n=mapping.find(question->getID());
		{
			if(m->first==n->first)
			{
				continue;
			}
			bool fragmentEdge=fragment->isEdge(m->first,n->first);
			bool databaseEdge=isEdge(m->second,n->second);
			if(fragmentEdge && ! databaseEdge)
			{
				return -1;
			}
			if(!fragmentEdge && databaseEdge )
			{
				d+=1.0;
			}
		}
		{
			bool fragmentEdge=fragment->isEdge(n->first,m->first);
			bool databaseEdge=isEdge(n->second,m->second);
			if(fragmentEdge && ! databaseEdge)
			{
				return -1;
			}
			if(!fragmentEdge && databaseEdge )
			{
				d+=1.0;
			}
		}
	}
	return d;
}

unsigned int MyGraph::mapItPartFrag(const MyGraph* fragment,std::map<unsigned int, mappingType > bags,std::map<unsigned int, unsigned int> mapping,unsigned int penalty,std::vector<unsigned int> order,MyPartitioning& iterPart)
{
	unsigned int count=0;
	unsigned int next=order[order.size()-1];
	order.pop_back();
	for(mappingType::iterator itr=bags[next].begin();
		itr!=bags[next].end(); itr++)
	{//for each partition
		for(std::vector<MyNode*>::iterator jtr=iterPart.getPartition(itr->first)->nodes.begin();
			jtr!=iterPart.getPartition(itr->first)->nodes.end(); jtr++)
		{//for each node in partition
			//find out if there are any mistakes
			bool badmapping=false;
			//std::cerr<<"Mapping: "<<next<<" -> "<<(*jtr)->getID()<<"\n";
			//unsigned int tempid=0,tempid2;
			for(std::map<unsigned int, unsigned int>::iterator m=mapping.begin();
				m!=mapping.end();m++)
			{
				/*
				tempid=m->first;
				tempid2=m->second;
				if( m->second==(*jtr)->getID())
				{
					fprintf(stderr,"bad 1\n");
				}
				else if(fragment->isEdge(next,m->first) && !isEdge((*jtr)->getID(),m->second))
				{fprintf(stderr,"bad 2\n");
				}
				else if(fragment->isEdge(m->first,next) && !isEdge(m->second,(*jtr)->getID()))
				{fprintf(stderr,"bad 3\n");
				}*/
				if( m->second==(*jtr)->getID()
					||(fragment->isEdge(next,m->first) && !isEdge((*jtr)->getID(),m->second))
					||(fragment->isEdge(m->first,next) && !isEdge(m->second,(*jtr)->getID())))
				{
					badmapping=true;
				}
			}
			//there were no mistakes
			if(badmapping==false)
			{
				mapping[next]=(*jtr)->getID();
				//std::cerr<<"Good."<<fragment->isEdge(next,tempid)<<" "<<isEdge((*jtr)->getID(),tempid2)<<" "<<fragment->isEdge(tempid,next)<<" "<<isEdge(tempid2,(*jtr)->getID())<<"\n";
				//std::cerr<<"Mapping: "<<next<<" -> "<<mapping[next]<<"\n";
				if(order.size()!=0)
				{
					count+=mapItPartFrag(fragment,bags,mapping,penalty,order,iterPart);
				}
				else
				{
					count++;
				}
			}
		}
	}
	//std::cerr<<"Returning: "<<count<<"\n";
	return count;
}
void MyGraph::compareItPartMappings(std::list<std::map<unsigned int,unsigned int> > maps)
{
	for(std::list<std::map<unsigned int, unsigned int> >::iterator 
		itr=maps.begin();itr!=maps.end();itr++)
	{//for each mapping
		std::list<std::map<unsigned int, unsigned int> >::iterator jtr=itr;
		jtr++;
		fprintf(stderr,"\n");
		for(std::map<unsigned int, unsigned int>::iterator ktr=itr->begin();ktr!=itr->end();ktr++)
		{
			fprintf(stderr,"%d\n",ktr->second);
		}
		for(;jtr!=maps.end();jtr++)
		{//for each mapping after that one
			std::map<unsigned int, bool> intersectionSet;
			std::map<unsigned int, bool> unionSet;
			//calculator union and intersection sets
			for(std::map<unsigned int, unsigned int>::iterator ktr=itr->begin();ktr!=itr->end();ktr++)
			{
				unionSet[ktr->second]=true;
			}
			for(std::map<unsigned int, unsigned int>::iterator ktr=jtr->begin();ktr!=jtr->end();ktr++)
			{
				unionSet[ktr->second]=true;
				if(itr->find(ktr->second)!=itr->end())
				{
					intersectionSet[ktr->second]=true;
				}
			}
			fprintf(stdout,"%f\n",((float) intersectionSet.size())/((float) unionSet.size()));
			
		}
		/*fprintf(stdout,"%f\n",(float intersection.size())/(float unionSet.size()));*/
	}
}

std::list<std::map<unsigned int,unsigned int> > MyGraph::mapItPartGreedy(const MyGraph* fragment,bagsType bags,unsigned int penalty,std::vector<unsigned int> order, unsigned int centerOrder,MyPartitioning& iterPart, arguments& args, std::map<unsigned int, std::vector<unsigned int> > &restrictions)
{//performs a very greedy mapping
	bool useRestrictions=(restrictions.size()>0);

	unsigned nodesToMap=fragment->getNumNodes();
	std::list<std::map<unsigned int,unsigned int> > returnVal;
	std::map<unsigned int, unsigned int> bestMap;
	unsigned int maxGreedyMaps=10;
	unsigned int i=0;
	for(i=0; i<nodesToMap && returnVal.size()<maxGreedyMaps ;i++)
	{
		if(bags[order[i]].size()==1 && iterPart.getPartition(bags[order[i]].begin()->first)->nodes.size()==1)
		{//singleton
			bestMap[order[i]]=iterPart.getPartition(bags[order[i]].begin()->first)->nodes[0]->getID();
		}
		else
		{
			unsigned int optionCount=0;
			for(mappingType::iterator j=bags[order[i]].begin();j!=bags[order[i]].end() && returnVal.size()<maxGreedyMaps ;j++)
			{
				//optionCount+=iterPart.getPartition(j->first)->nodes.size();
				for(std::vector<MyNode*>::iterator k=iterPart.getPartition(j->first)->nodes.begin();k< iterPart.getPartition(j->first)->nodes.end() && returnVal.size()<maxGreedyMaps ;k++)
				{
					if(useRestrictions 
						&& restrictions.find(order[i])!=restrictions.end() 
						&& std::find(restrictions[order[i]].begin(),restrictions[order[i]].end(),(*k)->getID())==restrictions[order[i]].end()
						)
					{continue;
					}
					optionCount+=1;
				}
			}
			if(optionCount>1000){fprintf(stderr,"too many options %d\n",optionCount);return returnVal;}
			for(mappingType::iterator j=bags[order[i]].begin();j!=bags[order[i]].end() && returnVal.size()<maxGreedyMaps ;j++)
			{
				for(std::vector<MyNode*>::iterator k=iterPart.getPartition(j->first)->nodes.begin();k< iterPart.getPartition(j->first)->nodes.end() && returnVal.size()<maxGreedyMaps ;k++)
				{
					if(useRestrictions 
						&& restrictions.find(order[i])!=restrictions.end() 
						&& std::find(restrictions[order[i]].begin(),restrictions[order[i]].end(),(*k)->getID())==restrictions[order[i]].end()
						)
					{
						continue;
					}
					//for each node possibility
					bestMap[order[i]]=(*k)->getID();
					returnVal.push_back(bestMap);
				}
			}
			break;
		}
	}
	fprintf(stderr,"%d possible maps\n",returnVal.size());
	unsigned int initialI=i+1;
	for(std::list<std::map<unsigned int, unsigned int> >::iterator theMapping=returnVal.begin();theMapping!=returnVal.end();theMapping++)
{
	bestMap=(*theMapping);


	for(i=initialI; i<nodesToMap;i++)
	{//fprintf(stderr,"Node number %d/%d\n",i,nodesToMap);
		float bestScore=-10;
		for(mappingType::iterator j=bags[order[i]].begin();j!=bags[order[i]].end();j++)
		{//fprintf(stderr,"-BagNumber %d\n",bags[order[i]].size());
			for(std::vector<MyNode*>::iterator k=iterPart.getPartition(j->first)->nodes.begin();k< iterPart.getPartition(j->first)->nodes.end();k++)
			{
				if(useRestrictions 
					&& restrictions.find(order[i])!=restrictions.end() 
					&& std::find(restrictions[order[i]].begin(),restrictions[order[i]].end(),(*k)->getID())==restrictions[order[i]].end()
					)
				{
					fprintf(stderr,"restricted %d - %d\n",order[i],(*k)->getID());
					continue;
				}
				if(bestMap.find(order[i])!=bestMap.end())
				{
					std::map<unsigned int, unsigned int> newTempMap=bestMap;
					newTempMap[order[i]]=(*k)->getID();
					double tempScore=verifyEdges(fragment,newTempMap,args,(*k));
					if(tempScore> bestScore)
					{
						bestScore=tempScore;
						bestMap=newTempMap;
					}
				}
				else
				{
					bestMap[order[i]]=(*k)->getID();
					bestScore=verifyEdges(fragment,bestMap,args,(*k));
				}
			}
		}
	}
	(*theMapping)=bestMap;
}
	return returnVal;
}
std::list<std::map<unsigned int,unsigned int> > MyGraph::mapItPartBFS(const MyGraph* fragment,bagsType bags,unsigned int penalty,std::vector<unsigned int> order, unsigned int centerOrder,MyPartitioning& iterPart)
{
	unsigned nodesToMap=fragment->getNumNodes();
	const unsigned long maxMaps=99999999;
	const unsigned largeError=99999999;
	const bool mapAll1s=false;
	std::map<unsigned int,std::list<std::map<unsigned int, unsigned int> > > mappings;
	std::list<std::map<unsigned int,unsigned int> > tempmaps;
fprintf(stdout,"or = %d\n",centerOrder);
	for(unsigned int i=0; i< nodesToMap;i++)//map all nodes
	{
		if(mapAll1s==true && i>0 && order[i-1]==centerOrder)
		{
			return mappings[i-1];
		}
		/*if( order[i]!=centerOrder &&(bags[order[i]].size()!=1
			 || iterPart.getPartition(bags[order[i]].begin()->first)->nodes.size()>1) )
		{fprintf(stdout,"a\n");
			if(i==0)
			{
				return tempmaps;
			}
			//fprintf(stdout,"copy %d [%d]\n",mappings[i-1].size(),i-1);
			tempmaps.assign(mappings[i-1].begin(),mappings[i-1].end());
			mappings[i]=tempmaps;
			//fprintf(stdout,"result %d [%d]\n",mappings[i].size(),i);
			if(centerOrder==order[i])
			{
				nodesToMap=i+1;
			}
			continue;
		}*/
		if(i==0)
		{
			fprintf(stderr,"Mapping %d\n",order[0]);
			if(bags[order[0]].size()>maxMaps*2)
			{
				std::cerr<<"Too many starting maps\n";
				continue;
			}
			else if(bags[order[0]].size()==0)
			{
				std::cerr<<"0 starting maps\n";
				continue;
			}
			std::map<unsigned int, unsigned int> mapping;
			tempmaps=mapItPartBFS2(fragment,bags,mapping,penalty,order[0],0,iterPart);
			mappings[i]=tempmaps;
		}
		if(i!=0)
		{
			if(mappings[i-1].size()>maxMaps || mappings[i-1].size()==0)
			{
				std::cerr<<"Counts:";
				//std::ofstream output2("Counts.txt",std::ios::out | std::ios::app);
				unsigned int count1;
				for(count1=0;count1<i-1;count1++)
				{
					std::cerr<<" "<<mappings[count1].size();
					//output2<<mappings[count1].size()<<", ";
				}
				if(mappings[i-1].size()==0)
				{
					for(;count1<nodesToMap;count1++)
					{
						std::cerr<<" "<<-1;
						//output2<<-1<<", ";
					}
				}
				else
				{
					for(;count1<nodesToMap;count1++)
					{
						std::cerr<<" "<<largeError;
						//output2<<largeError<<", ";
					}
				}
				//output2<<"\n";
				std::cerr<<std::endl;
				//output2.close();
				break;
			}
			//for each potential mapping on the previous level
			for(std::list<std::map<unsigned int, unsigned int> >::iterator mapping=mappings[i-1].begin();
				mapping!=mappings[i-1].end();mapping++)
			{
				/*if(i==1)
				{
					std::map<unsigned int, unsigned int>::iterator myitr=mapping->begin();
					fprintf(stderr,"y0=\n");
					printDegree(getNodeById(myitr->second),stderr);//#4
				}*/
				//this map needs to be extended. If any node has 0 viable options, then don't map.
				bool done=false;
				std::list<std::map<unsigned int,unsigned int> > bestTempMaps;
				//for each potential node in order, 
				for(unsigned int j=0;j<order.size()&&done==false;j++)
				{//that isn't in mapping, 
					if(mapping->find(order[j])==mapping->end())
					{//that has a neighbor 
						const edgelist* outedges=fragment->getNodeByIdConst(order[j])->getOutEdgesConst();
						bool wasAdj=false;
						for(edgelist::const_iterator N=outedges->begin();
							N!=outedges->end()&&done==false;N++)
						{//which is in the mapping:
							if(mapping->find(N->first)!=mapping->end())
							{
								wasAdj=true;
								//then this is a potential next node to map
								tempmaps=mapItPartBFS2(fragment,bags,(*mapping),0,order[j],bestTempMaps.size(),iterPart);
								/*if(i==1)
								{//#5
									unsigned int numopts=0;
									for(std::map<MyPartition*,unsigned int>::iterator oo=bags[order[j]].begin();
										oo!=bags[order[j]].end();oo++)
									{
										numopts+=oo->first->nodes.size();
									}
									fprintf(stderr,"|E(%d)|/|M(%d)| = %d / %d\n",(*N)->getID(),order[j], tempmaps.size(),numopts);
									//#6
								}*/
								if(tempmaps.size()>0)
								{
									if(bestTempMaps.size()==0 || tempmaps.size()<bestTempMaps.size())
									{
										bestTempMaps=tempmaps;
									}
								}
								else
								{//DROP!
									done=true;
								}
							}
						}
						if(!wasAdj && !done)
						{//this node was not adjacent
							if(bags[order[j]].size()<bestTempMaps.size() || bestTempMaps.size()==0)
							{
								tempmaps=mapItPartBFS2(fragment,bags,(*mapping),0,order[j],bestTempMaps.size(),iterPart);
								if(tempmaps.size()==0)
								{
									std::cerr<<"non-adjacent node had 0 mappings?\n";
								}
								bestTempMaps=tempmaps;
								//fprintf(stderr,"mapped %d\n",order[j]);
							}
						}
					}
				}
				if(done)
				{
					continue;
				}
				//since we didn't find a node which had 0 mappings, splice and continue!
				//we now have the besttempmaps
				mappings[i].splice(mappings[i].end(),bestTempMaps);
			}
		}
		//fprintf(stderr,"Number of tempmaps: %d\n",mappings[i].size());
	}
	std::cout<<"Counts:";
	//std::ofstream output2("Counts.txt",std::ios::out | std::ios::app);
	for(unsigned int count1=0;count1<nodesToMap;count1++)
	{
		std::cout<<" "<<mappings[count1].size();
		//output2<<mappings[count1].size()<<", ";
	}
	std::cout<<"\n";
	//output2<<"\n";
	//output2.close();
	return mappings[nodesToMap-1];
}
std::list<std::map<unsigned int,unsigned int> > MyGraph::mapItPartBFS2(const MyGraph* fragment,const bagsType &bags,std::map<unsigned int, unsigned int> mapping,unsigned int penalty,unsigned int next,unsigned int min,MyPartitioning& iterPart)
{
	std::list<std::map<unsigned int,unsigned int> > tempmaps;//return value
	bagsType::const_iterator nex=bags.find(next);
	for(mappingType::const_iterator itr=nex->second.begin();
		itr!=nex->second.end(); itr++)
	{//for each partition that "next" could be mapped to
		for(std::vector<MyNode*>::iterator jtr=iterPart.getPartition(itr->first)->nodes.begin();
			jtr!=iterPart.getPartition(itr->first)->nodes.end(); jtr++)
		{//for each node in partition
			//find out if there are any mistakes
			bool badmapping=false;
			//std::cerr<<"Mapping: "<<next<<" -> "<<(*jtr)->getID()<<"\n";
			//unsigned int tempid=0,tempid2;
			//check mapping
			for(std::map<unsigned int, unsigned int>::iterator m=mapping.begin();
				m!=mapping.end();m++)
			{
				/*
				tempid=m->first;
				tempid2=m->second;
				if( m->second==(*jtr)->getID())
				{
					fprintf(stderr,"bad 1\n");
				}
				else if(fragment->isEdge(next,m->first) && !isEdge((*jtr)->getID(),m->second))
				{fprintf(stderr,"bad 2\n");
				}
				else if(fragment->isEdge(m->first,next) && !isEdge(m->second,(*jtr)->getID()))
				{fprintf(stderr,"bad 3\n");
				}*/
				if( m->second==(*jtr)->getID()
					||(fragment->isEdge(next,m->first) && !isEdge((*jtr)->getID(),m->second))
					||(fragment->isEdge(m->first,next) && !isEdge(m->second,(*jtr)->getID())))
				{
					badmapping=true;
				}
			}
			//there were no mistakes
			if(badmapping==false)
			{
				mapping[next]=(*jtr)->getID();
				tempmaps.push_back(mapping);
				
				if(min!=0 && tempmaps.size()>min)
				{
					//std::cerr<<"Too many "<<tempmaps.size()<<std::endl;
					return tempmaps;
				}
			}
		}
	}
	//std::cerr<<"Returning: "<<tempmaps.size()<<"\n";
	return tempmaps;
}

//searches for the nBest matchs to the center such that each match has an adjacent node of the same type as neighbor
//and the sum( i-similarity(chainLink[], candidates[]) is minimized
//chainLinks is the list of nodes such that chainLink[i] is unique and there is an edge chainLink[i]->chainLink[i+1]
////currently limited to chainLinks.size == 3
//std::vector<unsigned int> Graph::find_by_chain(std::vector<const Node *> chainLinks, char similarityMethod, unsigned int i_degree, unsigned int nBest)
//{
//	std::vector<std::pair<double, unsigned int> > candidates;
//	double d;
//
//	if(chainLinks.size() != 3)
//	{
//		fprintf(stderr, "\nERROR: chainLinks.size() = %u", (unsigned int)chainLinks.size() );
//		return std::vector<unsigned int>();
//	}
//
//	candidates.reserve(nBest);
//
//	unsigned int startPartition=0, endPartition=nodeSorted.size();
//	getStartEndPartition(startPartition,endPartition,chainLinks[0]);
//
//	std::vector<const Node *> limitations; 
//	limitations.push_back(chainLinks[0]);
//	unsigned int i_less1, i_less2;
//	
//	i_less1 = i_degree -1;
//	if(i_less1 < 1)
//	{
//		i_less1 = 1;
//	}
//	i_less2 = i_less1 -1;
//	if(i_less2 < 1)
//	{ 
//		i_less2 = 1;
//	}
//
//	for(unsigned int i = startPartition; i < endPartition; ++i)
//	{
//		if(nodeSorted[i]->getType() != chainLinks[0]->getType() || nodeSorted[i]->getNumOutNeighbors() <= FEW_EDGES || nodeSorted[i]->getNumOutNeighbors() >= MANY_EDGES)
//		{
//			continue;
//		}
//
//		d = -1;
//
//
//		for(unsigned int j = 0; j < this->nodeSorted[i]->getNumOutNeighbors(); ++j)
//		{
//			const Node * n = this->getNodeById(this->nodeSorted[i]->outgoing_edges[j]);
//			float next = similarity(n, chainLinks[1], similarityMethod,i_less1);
//				
//			if(next >=0)
//			{
//				std::pair<const Node *, float> best = this->findBestMatchingNeighbor(n, chainLinks[2], limitations, similarityMethod, i_less2);
//				if(best.first != NULL && best.second >= 0)
//				{
//					float currentD = best.second + next;
//					if(currentD < d || d == -1)
//					{
//						d = currentD;
//					}
//				}
//			}
//		}
//
//		if(d < 0)
//		{
//			continue;
//		}
//
//		float temp = similarity(this->nodeSorted[i], chainLinks[0], similarityMethod, i_degree);
//		if(temp < 0)
//		{
//			continue;
//		}
//		d+=temp;
//		updateBestCandidates(candidates, std::pair<double,unsigned int>(d,nodeSorted[i]->getID()), nBest);
//	}
//
//
//	std::vector<unsigned int> result(nBest);
//	for(unsigned int itr=0;itr<candidates.size();itr++)
//	{
//		result[itr]=candidates[itr].second;
//	}
//	
//	return result;
//}



/* fragment generation methods */
//all methods for generating fragments MUST add as possible types the entire type vector of the original graph to allow for proper comparison of i-degrees

//generates a fragment by a probabilistic-random walk. With a probability of p, the walk will restart
//if the walk restarts and no new nodes were discovered, p is updated to p = p*alpha
void MyGraph::generateFragment_pRandomWalk(MyGraph* database, unsigned int numnodes, float p, float alpha, unsigned int minEdges,arguments& args, bool useBackEdges)
{
	unsigned int candidate_index = 0;
	unsigned int candidate_edge_count = 0;
	do
	{
		candidate_index = rand()%database->getNumNodes();
		candidate_edge_count = (database->getNodeByIndex(candidate_index))->getNumOutNeighbors();
	}
	while (candidate_edge_count >= MANY_EDGES || candidate_edge_count < minEdges) ;

	this->generateFragment_pRandomWalk(database, database->getNodeByIndex(candidate_index), numnodes, p, alpha, args,useBackEdges);
}
void MyGraph::generateFragment_pRandomWalk(MyGraph* database, const MyNode* center, unsigned int numnodes, float p, float alpha,arguments& args, bool useBackEdges)
{
	this->addNode(center, false);
	const MyNode * current = center;
	bool found = false;
	unsigned int restarts = 0;
	while(this->getNumNodes() < numnodes && restarts <= 40)//hardcoded...?
	{
		//go to the next node (if possible)
		edgelist alledges;
		bool deadEnd=false;
		if(useBackEdges && current->getNumAllNeighbors()>0)
		{
			alledges=current->getUndirectedEdges();
		}
		else if(!useBackEdges && current->getNumOutNeighbors() > 0)
		{ 	
			alledges.insert(current->getOutEdgesConst()->begin(),current->getOutEdgesConst()->end());
		}
		else
		{
			deadEnd=true;
		}
		if(!deadEnd){
			edgelist::iterator itr=alledges.begin();
			std::advance(itr,rand()%alledges.size());
			current = itr->second.getNode();//alledges[rand()%alledges.size()].getNode();
			found = (this->addNode(current, true)!= 0)||found ;
		}
		//check for restart
		if(deadEnd || (double)(rand())/(double)INT_MAX < p || current->getNumOutNeighbors() == 0)
		{
			if(!found)
			{
				p *= alpha;
			}
			current = center; 
			found = false;
			++restarts;
		}
	}
	//The fragment is done generating
	positioningArgument pos;
	initIndex(args.partitionMethod2,pos,args);
}

//restricted breadth first search to maximum of maxDist
bool MyGraph::rBFS(unsigned int startID, unsigned int targetID, unsigned int maxDist, int direction) const
{
	static unsigned int lastID = UINT_MAX;
	static unsigned int lastMax = 0;
	static std::map<unsigned int, unsigned int> visited_Out, visited_In;
	bool result = startID==targetID;

	
	if(lastID != startID || lastMax != maxDist)
	{
		//printf("\nrBFS(%u, %u, %u, %d)", startID, targetID, maxDist, direction);
	
		visited_Out.clear(); 
		visited_In.clear();

		if(direction >=0)
		{
			visited_Out = rBFS(startID, maxDist,true);
		}

		if(direction <= 0)
		{
			visited_In = rBFS(startID, maxDist, false);
		}
		lastID = startID;
		lastMax = maxDist;
	}

	result = visited_Out.find(targetID) != visited_Out.end() || visited_In.find(targetID) != visited_In.end();

	return result;
}

//returns all nodes within maxDist of startID and their distance from startID
std::map<unsigned int, unsigned int> MyGraph::rBFS(unsigned int startID, unsigned int maxDist, bool outgoing) const
{
	unsigned int dist = 0; 
	std::map<unsigned int, unsigned int> visited;
	std::queue<unsigned int> *nodesToVisit = new std::queue<unsigned int>;
	std::queue<unsigned int> *nextLayer = new std::queue<unsigned int>;
	const edgelist* neighbors;

	nodesToVisit->push(startID);
	visited[startID] = 0;

	dist = 1;
	while(dist <= maxDist && !nodesToVisit->empty())
	{
		while(!nodesToVisit->empty())
		{
			unsigned int nextID = nodesToVisit->front();
			nodesToVisit->pop();
			visited[nextID] = dist-1;
			if(outgoing)
			{
				neighbors = getNodeByIdConst(nextID)->getOutEdgesConst();
			}
			else
			{
				neighbors = getNodeByIdConst(nextID)->getInEdgesConst();
				//fprintf(stderr,"rBFS is not implemented for outgoing=false\n");
			}
			for(edgelist::const_iterator i=neighbors->begin(); i !=neighbors->end(); ++i)
			{
				if(visited.find(i->first) == visited.end())
				{
					nextLayer->push(i->first); 
					visited[i->first] = dist;
				}
			}
		}

		//swap them!
		std::queue<unsigned int> * temp = nextLayer;
		nextLayer = nodesToVisit;
		nodesToVisit = temp;
		++dist;
	}

	delete nodesToVisit;
	delete nextLayer;

	return visited;
}

/*locality*/

//computes the local positions of all nodes based on the locality
//locality = 1,2,3 positions are updated based on a distance of (locality) from v
//locality = -1 positions are updated based on a random walk with returns
void MyGraph::localPositions(int locality,const arguments& args)
{
	if(locality >=3)
	{
		fprintf(stderr, "calculating 3-neighborhood locality\n");
		this->updatePositions(1,args);
		fprintf(stderr, "done calculating 3-neighborhood locality");
		return;
	}
	else if(locality==-1)
	{
		fprintf(stderr,"localPositions(-1) not implemented\n");
	}
	else
	{
		fprintf(stderr, "\ncalling positionByNeighborhood(%d)", locality);
		positionedByNeighborhood(locality,args);
	}
}

void MyGraph::positionedByNeighborhood(int n,const arguments& args)
{
	for(mapIDNodes::iterator itr=nodes.begin();itr!=nodes.end();itr++)
	{
		//fprintf(stderr, "\n\t%s(%u) outdegree(%u)", itr->second->getLabel(), itr->second->getID(), itr->second->getNumOutNeighbors());
		//fflush(stderr);

		MyGraph* temp=getSubgraphToDegreeById(itr->second.getID(), n, true,args,true);//run a minimalist degreeupdate
		
		/*fprintf(stderr, "\n\tsubgraph size(%u)", temp->getNumNodes() );
		fflush (stderr);*/

		for(unsigned int i=0;i<3;i++)
		{
			const MyNode * n = temp->getNodeById(itr->second.getID());
			//itr->second->iDegree[i] = n->iDegree[i];
			itr->second.getIDegreeVar(i+1,false)->setList(*(n->getIDegree(i+1,false)->getList()));
		}
		
		/*fprintf(stderr, "\n\ti-degrees complete");
		fflush(stderr);*/

		delete temp;
	}

	fprintf(stderr, "\ncompleted positioning");
	fflush(stderr);
}

/*destructor*/
MyGraph::~MyGraph()
{
}

/* helper methods */
//creates the actual node if possible
unsigned int MyGraph::createNode(unsigned char llen, char* label, unsigned short type, unsigned int id)
{
	if(this->nodes.find(id) != this->nodes.end() )
	{
		return 0;
	}

	struct MyNode n(id,type,label,llen);
	this->nodes[id] = n;

	if(this->getNumNodes()==1)
	{
		originator = id;
	}
	this->uptodatePartitions = false;

	//fprintf(stderr, "\nSuccessfully added node %s(%u)", n->getLabel(), n->getID());
	return 1; //returns non zero to ensure that the node was successfully created
}


//calculates the i-degree of node <id> using <method> method = 1: sum of types, method = 0, probability of ending at a type
void MyGraph::calculateIDegree(unsigned int id, int method, unsigned int i)
{
	if(method!=1)
	{
		fprintf(stderr,"calculateIDegree, method=%d not implemented\n",method);
		return;
	}
	MyNode *n = getNodeById(id);

	if( i > 3 || i < 1)
	{
		fprintf(stderr, "\nError: cannot calculate %u-degree of node %s(%u)", i, n->getLabel(), n->getID());
		fflush(stderr);
		return;
	}

	//fprintf(stderr, "\n\tresizing the %udegree vector: %s(%u)", i, n->getLabel(), n->getID());
	//fflush(stderr);

	n->getIDegreeVar(i,false)->getListVar()->clear();
	n->getIDegreeVar(i,true)->getListVar()->clear();
	//n->iDegree[i-1].clear();
	//n->iDegree[i-1].resize(this->types.size(), 0);
	
	//n->iniDegree[i-1].clear();
	//n->iniDegree[i-1].resize(this->types.size(), 0);
	//fprintf(stderr, "\n\tsuccess resize(%u)", n->iDegree[i-1].size());
	//fflush(stderr);

	if(i == 1)
	{
		//fprintf(stderr, "\n\tbegin 1-deg calculation");
		//fflush(stderr);
		edgelist* outedges=n->getOutEdges();
		for(edgelist::iterator j=outedges->begin(); j!=outedges->end(); ++j)
		{
			unsigned int next = j->first;
			//n->iDegree[i-1][t]++; //increase the count of this type
			n->getIDegreeVar(i,false)->append(getNodeById(next)->getType(),1);
		}
		edgelist* inedges=n->getInEdges();
		for(edgelist::iterator j=inedges->begin(); j !=inedges->end(); ++j)
		{
			unsigned int next = j->first;
			//n->iniDegree[i-1][t]++; //increase the count of this type
			n->getIDegreeVar(i,true)->append(getNodeById(next)->getType(),1);
		}
		//fprintf(stderr, " end 1-deg calculation");
	}
	else if( i > 1)
	{
		edgelist* outedges=n->getOutEdges();
		for(edgelist::iterator j=outedges->begin(); j !=outedges->end(); ++j)
		{
			unsigned int next = j->first;
			//std::vector<float> next_ideg = this->nodes[next]->iDegree[i-2]; //subtract two because iDeg[j] is the j+1 degree and we want the i-1 degree (so if i=2 we want 1 in which case j = 0)
			const iDegreeList* next_ideg = getNodeById(next)->getIDegree(i-1,false)->getList();
			/*if(next_ideg.size() != n->iDegree[i-1].size())
			{
				fprintf(stderr, "\nWOAH: next_ideg.size() [%u] != i_deg.size()[%u]", (unsigned)next_ideg.size(), (unsigned)n->iDegree[i-1].size() );
				break;
			}*/
			for(iDegreeList::const_iterator itr=next_ideg->begin();itr!=next_ideg->end();itr++)
			{
				n->getIDegreeVar(i,false)->append(itr->first,itr->second);
			}
			/*for(unsigned int k = 0; k < n->iDegree[i-1].size(); ++k)
			{
				n->iDegree[i-1][k] += next_ideg[k];
			}*/
		}
		edgelist* inedges=n->getInEdges();
		for(edgelist::iterator j=inedges->begin(); j != inedges->end(); ++j)
		{
			unsigned int next = j->first;
			//std::vector<float> next_ideg = this->nodes[next]->iniDegree[i-2]; //subtract two because iDeg[j] is the j+1 degree and we want the i-1 degree (so if i=2 we want 1 in which case j = 0)
			const iDegreeList* next_ideg = getNodeById(next)->getIDegree(i-1,true)->getList();
	
			/*if(next_ideg.size() != n->iniDegree[i-1].size())
			{
				fprintf(stderr, "\nWOAH: next_ideg.size() [%u] != i_deg.size()[%u]", (unsigned)next_ideg.size(), (unsigned)n->iniDegree[i-1].size() );
				break;
			}*/
			for(iDegreeList::const_iterator itr=next_ideg->begin();itr!=next_ideg->end();itr++)
			{
				n->getIDegreeVar(i,true)->append(itr->first,itr->second);
			}

			/*for(unsigned int k = 0; k < n->iniDegree[i-1].size(); ++k)
			{
				n->iniDegree[i-1][k] += next_ideg[k];
			}*/
		}
	}
}//calculateIDegree


unsigned int MyGraph::partitionStats(stats_keeper stats[], unsigned int toofew)
{
	unsigned int result = 0;
	unsigned int size=0;
	
	for(unsigned int j = 0; j < this->nodeSorted.size(); ++j)
	{
		if(this->nodeSorted[j]->getNumOutNeighbors() >= toofew)
		{
			++size;
		}
		else
		{
			++result;
		}

		if(j == this->nodeSorted.size()-1 || this->nodeSorted[j]->getType() != this->nodeSorted[j+1]->getType())
		{
			if(size ==0)
			{
				fprintf(stderr, "\nupdating 0-partition stats type(%u) size(%u)", this->nodeSorted[j]->getType(), size);
				fflush(stderr);
			}
			stats[0].update(size);
			size = 0;
		}
	}

	for(unsigned int i = 1; i < 7; ++i)
	{

		for(unsigned int j = 0; j < this->partitionStarts[i].size()-1; ++j)
		{
			size = this->partitionStarts[i][j+1]-this->partitionStarts[i][j];
			
			if(this->nodeSorted[this->partitionStarts[i][j]]->getNumOutNeighbors() < toofew)
			{
					continue;
			}
			
			stats[i].update(size);
		}

		//get the last partitions info
		if(this->partitionStarts[i].size() > 0 && 
			this->nodeSorted[this->partitionStarts[i][this->partitionStarts[i].size()-1]]->getNumOutNeighbors() >= toofew)
		{
			size = this->nodeSorted.size() - this->partitionStarts[i][this->partitionStarts[i].size() -1];
			stats[i].update(size);
		}
	}

	return result;
}//partition stats


//fuzzyDiffers should be an array of size maxDiffers+1
void MyGraph::fuzzyStats(unsigned int i_deg, unsigned int maxDiffers, unsigned int toofew, unsigned int samples,stats_keeper *fuzzydiffers, stats_keeper& avgFuzziness, char distMethod,struct arguments args )
{
	std::vector<unsigned int> sampledIDs(samples, INT_MAX);//ensure that no partition is tested twice
	unsigned int id;
	bool newPart;
	MyNode * n, *n2;
	std::vector<unsigned int> suitable;


	suitable.reserve(this->partitionStarts[i_deg].size() );
	for(unsigned int i = 0; i < this->partitionStarts[i_deg].size(); ++i)
	{
		if(this->nodeSorted[this->partitionStarts[i_deg][i]]->getNumOutNeighbors() >= toofew)
		{
			suitable.push_back(this->partitionStarts[i_deg][i]);
		}
	}


	std::vector<unsigned int> fuzzyNodes;
	double avgFuz;
	
	printf("\nnumber of samples: %u", samples);
	for(unsigned int test = 0; test < samples; ++test)
	{
		avgFuz = 0; 
		fuzzyNodes.clear();
		fuzzyNodes.resize(maxDiffers+1, 0);
		

		//generate a random node to compare
		newPart = false;
		while(!newPart)
		{
			id = rand() % suitable.size();
			n = this->nodeSorted[suitable[id]];
			//mark that this ID was used 
			sampledIDs[test] = suitable[id];

			newPart = true;
			//check that it doesn't match any of the previously checked 
			for(unsigned int i = 0; i < test && newPart; ++i)
			{
				n2 = this->nodeSorted[sampledIDs[i]];
				newPart = (dist(n, n2, i_deg,distMethod,args) != 0);
			}
		}//newpart

		unsigned int start=0, end=this->nodeSorted.size();
		for(size_t k = 0; k < this->partitionStarts[0].size(); ++k)
		{
			if(this->nodeSorted[this->partitionStarts[0][k]]->getType() == n->getType())
			{
				start = this->partitionStarts[0][k];
				if(k <this->partitionStarts[0].size() -1)
				{
					end = this->partitionStarts[0][k+1];
				}
				break;
			}
		}
		unsigned int count = 0;
		for(size_t k = start; k < end; ++k)
		{
			if(this->nodeSorted[k]->getNumOutNeighbors() < toofew ||this->nodeSorted[k]->getType() != n->getType())
			{
				continue;
			}

			n2 = this->nodeSorted[k];
			int d;
			d = (int)dist(n, n2, i_deg,distMethod,args);
			if(d >=0)
			{
				avgFuz+= d;
				count++;
				for(unsigned int l = 0; l < fuzzyNodes.size(); ++l)
				{
					if(d <= (long int)l)
					{
						fuzzyNodes[l]++;
					}
				}
			}
		}
		//update statistics
		avgFuz /= count;
		avgFuzziness.update(avgFuz);
		for(unsigned int l = 0; l < maxDiffers+1; ++l)
		{
			fuzzydiffers[l].update(fuzzyNodes[l]);
		}
	}//fuzzy samples
}

std::vector< std::vector<unsigned int > > MyGraph::connectedComponents()
{
	unsigned int color = 0; 
	mapIDNodes::iterator current;
	std::map<unsigned int, unsigned int> visited;
	std::queue<unsigned int> *nodesToVisit = new std::queue<unsigned int>;
	std::queue<unsigned int> *nextLayer = new std::queue<unsigned int>;
	
	std::vector< std::vector<unsigned int> > components;
	
	for(current = this->nodes.begin(); current != this->nodes.end(); current++)
	{
		if(visited.find(current->second.getID())!=visited.end())
		{
			continue;
		}
		std::vector<unsigned int> a;
		components.push_back(a);
		nodesToVisit->push(current->second.getID());
		while(!nodesToVisit->empty())
		{
			while(!nodesToVisit->empty())
			{
				unsigned int nextID = nodesToVisit->front();
				nodesToVisit->pop();
				if(visited.find(nextID)==visited.end())
				{
					visited[nextID] = color;
					components[color].push_back(nextID);

					edgelist* neighbors = getNodeById(nextID)->getOutEdges();
					for(edgelist::iterator i=neighbors->begin(); i != neighbors->end(); ++i)
					{
						if(visited.find(i->first) == visited.end())
						{
							nextLayer->push(i->first);
						}
					}
					if(this->directed)
					{
						neighbors = getNodeById(nextID)->getInEdges();
						for(edgelist::iterator i=neighbors->begin(); i !=neighbors->end(); ++i)
						{
							if(visited.find(i->first) == visited.end())
							{
								nextLayer->push(i->first);
							}
						}
					}
				}
			}

			//swap them!
			std::queue<unsigned int> * temp = nextLayer;
			nextLayer = nodesToVisit;
			nodesToVisit = temp;
		}
		color++;
	}
	delete nodesToVisit;
	delete nextLayer;

	return components;
}
unsigned int MyGraph::Diameter()
{
	std::vector< std::vector<unsigned int > > components=connectedComponents();
	fprintf(stdout,"There are %d components:\n",(unsigned)components.size());
	for(unsigned int i=0;i<components.size();i++)
	{
		unsigned int diameter=0;
		for(unsigned int j=0;j<components[i].size();j++)
		{
			std::map<unsigned int, unsigned int> distMap=rBFS(components[i][j],components[i].size(),true);
			for(std::map<unsigned int,unsigned int>::iterator itr=distMap.begin();itr!=distMap.end();itr++)
			{
				if(itr->second > diameter)
				{
					diameter=itr->second;
				}
			}
			fprintf(stdout,"%f: Diameter updated to: %d\n",((float)j)/(float)components[i].size(),diameter);
		}
		fprintf(stdout,"Size:\t%d\tDia:\t%d\n",(unsigned)components[i].size(),diameter);
		fprintf(stderr,"Size:\t%d\tDia:\t%d\n",(unsigned)components[i].size(),diameter);
	}
	return 0;
}

//this node returns the most diverse neigbhor of the given node specified by id
const MyNode * MyGraph::getMostDiverseNeighbor(unsigned int id, unsigned int limits[] , unsigned int size)const
{
	const MyNode * result = NULL; 
	const MyNode * temp;
	const MyNode * n = this->getNodeByIdConst(id);
	unsigned int j = 0;

	//find first that's not limited
	const edgelist* outedges=n->getOutEdgesConst();
	for(edgelist::const_iterator j=outedges->begin(); j !=outedges->end() && result == NULL; j++)
	{
		bool cont = true;
		temp = j->second.getNode();
		for(unsigned int i = 0; i < size&& cont; i++)
		{
			cont = temp->getID() != limits[i];
		}
		if(cont)
		{
			if(result==NULL)
			{
				result = temp;
			}
			else
			{
				if(this->isMoreDiverse(temp, result) )
				{
					result = temp;
				}
			}
		}
	}

	return result;
}

//returns the number of different types found in the neighbors of the node
unsigned int MyGraph::diversity(const MyNode * n)const
{
	if(n == NULL)
	{
		return 0;
	}
	std::map<unsigned short, bool> typesSeen;
	unsigned int t;

	const edgelist* outedges=n->getOutEdgesConst();
	for(edgelist::const_iterator i=outedges->begin(); i !=outedges->end(); i++)
	{
		t = i->second.getNode()->getType();
		if(typesSeen.find(t)!=typesSeen.end())
		{
			typesSeen[t] = true;
		}
	}

	return typesSeen.size();
}
//returns the number of different types found in the neighbors of the node
unsigned int MyGraph::diversityTotal(const MyNode * n)const
{
	if(n == NULL)
	{
		return 0;
	}
	float returnVal=0;
	for(std::vector<IndexingNode*>::const_iterator itr=n->partitionPtrs.begin();
		itr!=n->partitionPtrs.end();itr++)
	{
		std::map<edgeDescriptor,float>* idegree=(*itr)->getIndexValue()->getIDegree();
		for(std::map<edgeDescriptor,float>::iterator iItr=idegree->begin();
			iItr!=idegree->end(); iItr++)
		{
			returnVal+=iItr->second;
		}
	}
	return (unsigned int) returnVal;
}

//returns true if diversity(l) > diversity (r) or if diversity(l) == diversity(r) AND l has more neighbors
//in the event that either node is NULL, the case of false is returned
bool MyGraph::isMoreDiverse(const MyNode * l, const MyNode * r)const
{
	bool result = false;

	if(l != NULL && r != NULL)
	{
		unsigned int d_l = diversity(l);
		unsigned int d_r = diversity(r);
		result = (d_l > d_r) || (d_l == d_r && l->getNumOutNeighbors() > r->getNumOutNeighbors() );
	}

	return result;
}	
bool MyGraph::isMoreDiverseTotal(const MyNode * l, const MyNode * r)const
{
	bool result = false;

	if(l != NULL && r != NULL)
	{
		unsigned int d_l = diversity(l);
		unsigned int d_r = diversity(r);
		result = (d_l > d_r) || (d_l == d_r && l->getNumAllNeighbors() > r->getNumAllNeighbors() );
	}

	return result;
}	

//frequencies should be a NULL pointer, when returned frequencies[A][B] will represent the frequency that type B is a neighbor of type A
bool MyGraph::frequencyOfNeighbors(std::vector<std::vector<float> > &frequencies) const
{
	fprintf(stderr,"frequencyOfNeighbors isn't implemented using the new idegree format\n");//TODO
	exit(1);
	/*
	unsigned int count = 0; 

	//initialize frequencies as a square matrix
	frequencies.clear();
	frequencies.resize(this->types.size(), std::vector<float>(this->types.size(), 0.0) );

	//check if there is any point in continuing
	if(this->types.size() != this->partitionStarts[0].size() || this->nodeSorted.size() == 0)
	{
		fprintf(stderr, "\nERROR: frequencyOfNeighbors types.size() [%u] != partitionStarts.size() [%u]", (unsigned int)this->types.size(), (unsigned int)this->partitionStarts[0].size());
		fflush(stderr);
		return false;
	}

	unsigned int t, previoust = this->normalizedType(this->nodeSorted[0]->getType());

	for(unsigned int i = 0; i < this->nodeSorted.size(); ++i)
	{
		t = this->normalizedType(this->nodeSorted[i]->getType());

		if(t != previoust && count != 0)
		{
			for(unsigned int k = 0; k < this->types.size(); ++k)
			{
				frequencies[previoust][k] /= count;
			}
				
			count = 0; 
		}

		std::vector<const MyNode * > neighbors = this->getNeighborsByIndex(i);
		for(unsigned int j = 0; j < neighbors.size(); ++j)
		{
			unsigned int neighborT = this->normalizedType(neighbors[j]->getType());
			frequencies[t][neighborT]++;
			++count;
		}

		previoust = t;
	}

	return true;*/
}

int MyGraph::setOverlap(const MyGraph* f)const
{
	int result = 0;


	mapIDNodes::const_iterator i = f->nodes.begin(),
		j = this->nodes.begin();

	while( i != f->nodes.end() && j != this->nodes.end() )
	{

		if(i->first == j->first)
		{
			result++;
			i++;
			j++;
		}
		else if(i->first < j->first)
		{
			i++;
		}
		else
		{
			j++;
		}
	}
	

	return result;
}



